#!/usr/bin/env python3
# pylint:disable=missing-class-docstring,no-self-use,wrong-import-order
__package__ = __package__ or "tests.analyses.cfg"  # pylint:disable=redefined-builtin

import os
import logging
import unittest

import archinfo
import angr
from angr.analyses.cfg.cfg_fast import SegmentList
from angr.knowledge_plugins.cfg import CFGNode, CFGModel, MemoryDataSort
from angr.analyses.cfg.indirect_jump_resolvers import mips_elf_fast

from ...common import bin_location, slow_test

l = logging.getLogger("angr.tests.test_cfgfast")

test_location = os.path.join(bin_location, "tests")


class TestCfgfast(unittest.TestCase):
    def cfg_fast_functions_check(self, arch, binary_path, func_addrs, func_features):
        """
        Generate a fast CFG on the given binary, and test if all specified functions are found

        :param str arch: the architecture, will be prepended to `binary_path`
        :param str binary_path: path to the binary under the architecture directory
        :param dict func_addrs: A collection of function addresses that should be recovered
        :param dict func_features: A collection of features for some of the functions
        :return: None
        """

        path = os.path.join(test_location, arch, binary_path)
        proj = angr.Project(path, load_options={"auto_load_libs": False})

        cfg = proj.analyses.CFGFast()
        assert set(cfg.kb.functions.keys()).issuperset(func_addrs)

        for func_addr, feature_dict in func_features.items():
            returning = feature_dict.get("returning", "undefined")
            if returning != "undefined":
                assert cfg.kb.functions.function(addr=func_addr).returning is returning

        # Segment only
        cfg = proj.analyses.CFGFast(force_segment=True)
        assert set(cfg.kb.functions.keys()).issuperset(func_addrs)

        for func_addr, feature_dict in func_features.items():
            returning = feature_dict.get("returning", "undefined")
            if returning != "undefined":
                assert cfg.kb.functions.function(addr=func_addr).returning is returning

        # with normalization enabled
        cfg = proj.analyses.CFGFast(force_segment=True, normalize=True)
        assert set(cfg.kb.functions.keys()).issuperset(func_addrs)

        for func_addr, feature_dict in func_features.items():
            returning = feature_dict.get("returning", "undefined")
            if returning != "undefined":
                assert cfg.kb.functions.function(addr=func_addr).returning is returning

    def cfg_fast_edges_check(self, arch, binary_path, edges):
        """
        Generate a fast CFG on the given binary, and test if all edges are found.

        :param str arch: the architecture, will be prepended to `binary_path`
        :param str binary_path: path to the binary under the architecture directory
        :param list edges: a list of edges
        :return: None
        """

        path = os.path.join(test_location, arch, binary_path)
        proj = angr.Project(path, load_options={"auto_load_libs": False})

        cfg = proj.analyses.CFGFast()

        for src, dst in edges:
            src_node = cfg.model.get_any_node(src)
            dst_node = cfg.model.get_any_node(dst)
            assert src_node is not None, "CFG node 0x%x is not found." % src
            assert dst_node is not None, "CFG node 0x%x is not found." % dst
            assert dst_node in src_node.successors, "CFG edge {}-{} is not found.".format(
                src_node,
                dst_node,
            )

    def test_cfg_0(self):
        functions = {
            0x400410,
            0x400420,
            0x400430,
            0x400440,
            0x400470,
            0x40052C,
            0x40053C,
        }

        function_features = {}

        self.cfg_fast_functions_check("x86_64", "cfg_0", functions, function_features)

    def test_cfg_0_pe(self):
        functions = {
            # 0x40150a,  # currently angr identifies 0x40150e due to the way _func_addrs_from_prologues() is
            # implemented. this issue can be resolved with a properly implemented approach like Byte-Weight
            0x4014F0,
        }

        function_features = {}

        self.cfg_fast_functions_check("x86_64", "cfg_0_pe", functions, function_features)

    def test_arm_function_merge(self):
        # function 0x7bb88 is created due to a data hint in another block. this function should be merged with the
        # previous function 0x7ba84

        path = os.path.join(test_location, "armel", "tenda-httpd")
        proj = angr.Project(path, auto_load_libs=False)

        cfg = proj.analyses.CFGFast()

        node_7bb88 = cfg.model.get_any_node(0x7BB88)
        assert node_7bb88 is not None
        assert node_7bb88.function_address == 0x7BA84

    @slow_test
    def test_busybox(self):
        edges = {
            (0x4091EC, 0x408DE0),
            # call to putenv. address of putenv may change in the future
            (
                0x449ACC,
                0x5003B8,
            ),
            # call to free. address of free may change in the future
            (
                0x467CFC,
                0x500014,
            ),
        }

        self.cfg_fast_edges_check("mipsel", "busybox", edges)

    @slow_test
    @unittest.skipUnless(
        os.path.isfile("C:\\Windows\\System32\\ntoskrnl.exe"),
        "ntoskrnl.exe does not exist on this system.",
    )
    def test_ntoskrnl(self):
        # we cannot distribute ntoskrnl.exe. as a result, this test case is manual
        path = "C:\\Windows\\System32\\ntoskrnl.exe"
        proj = angr.Project(path, auto_load_libs=False)
        _ = proj.analyses.CFG(data_references=True, normalize=True, show_progressbar=True)

        # nothing should prevent us from finish creating the CFG

    def test_fauxware_function_feauters_x86_64(self):
        functions = {
            0x4004E0,
            0x400510,
            0x400520,
            0x400530,
            0x400540,
            0x400550,
            0x400560,
            0x400570,  # .plt._exit
            0x400580,  # _start
            0x4005AC,
            0x4005D0,
            0x400640,
            0x400664,
            0x4006ED,
            0x4006FD,
            0x40071D,  # main
            0x4007E0,
            0x400870,
            0x400880,
            0x4008B8,
        }

        function_features = {
            0x400570: {"returning": False},  # plt.exit
            0x4006FD: {"returning": False},  # rejected
        }

        return_edges = {
            (0x4006FB, 0x4007C7),
        }  # return from accepted to main

        self.cfg_fast_functions_check("x86_64", "fauxware", functions, function_features)
        self.cfg_fast_edges_check("x86_64", "fauxware", return_edges)

    def test_fauxware_function_features_mips(self):
        functions = {
            0x400534,  # _init
            0x400574,
            0x400598,
            0x4005D0,  # _ftext
            0x4005DC,
            0x400630,  # __do_global_dtors_aux
            0x4006D4,  # frame_dummy
            0x400708,
            0x400710,  # authenticate
            0x400814,  # accepted
            0x400868,  # rejected
            0x4008C0,  # main
            0x400A34,
            0x400A48,  # __libc_csu_init
            0x400AF8,
            0x400B00,  # __do_global_ctors_aux
            0x400B58,
            ### plt entries
            0x400B60,  # strcmp
            0x400B70,  # read
            0x400B80,  # printf
            0x400B90,  # puts
            0x400BA0,  # exit
            0x400BB0,  # open
            0x400BC0,  # __libc_start_main
        }

        function_features = {
            0x400868: {  # rejected
                "returning": False,
            }
        }

        return_edges = {
            (0x40084C, 0x400A04),
        }  # returning edge from accepted to main

        self.cfg_fast_functions_check("mips", "fauxware", functions, function_features)
        self.cfg_fast_edges_check("mips", "fauxware", return_edges)

    def test_mips_elf_fast_indirect_jump_resolver(self):
        bin_path = os.path.join(test_location, "mips", "fauxware")
        proj = angr.Project(bin_path, auto_load_libs=False)
        # enable profiling for MipsElfFast
        # FIXME: The result might be different if other test cases that run in parallel mess with the profiling setting
        mips_elf_fast.enable_profiling()
        _ = proj.analyses.CFG()
        mips_elf_fast.disable_profiling()
        assert mips_elf_fast.HITS_CASE_0 >= 10

    def test_cfg_loop_unrolling(self):
        edges = {
            (0x400658, 0x400636),
            (0x400658, 0x400661),
            (0x400651, 0x400636),
            (0x400651, 0x400661),
        }

        self.cfg_fast_edges_check("x86_64", "cfg_loop_unrolling", edges)

    def test_cfg_switches_x86_64(self):
        edges = {
            # jump table 0 in func_0
            (0x40053A, 0x400547),
            (0x40053A, 0x400552),
            (0x40053A, 0x40055D),
            (0x40053A, 0x400568),
            (0x40053A, 0x400573),
            (0x40053A, 0x400580),
            (0x40053A, 0x40058D),
            # jump table 0 in func_1
            (0x4005BC, 0x4005C9),
            (0x4005BC, 0x4005D8),
            (0x4005BC, 0x4005E7),
            (0x4005BC, 0x4005F6),
            (0x4005BC, 0x400605),
            (0x4005BC, 0x400614),
            (0x4005BC, 0x400623),
            (0x4005BC, 0x400632),
            (0x4005BC, 0x40063E),
            (0x4005BC, 0x40064A),
            (0x4005BC, 0x4006B0),
            # jump table 1 in func_1
            (0x40065A, 0x400667),
            (0x40065A, 0x400673),
            (0x40065A, 0x40067F),
            (0x40065A, 0x40068B),
            (0x40065A, 0x400697),
            (0x40065A, 0x4006A3),
            # jump table 0 in main
            (0x4006E1, 0x4006EE),
            (0x4006E1, 0x4006FA),
            (0x4006E1, 0x40070B),
            (0x4006E1, 0x40071C),
            (0x4006E1, 0x40072D),
            (0x4006E1, 0x40073E),
            (0x4006E1, 0x40074F),
            (0x4006E1, 0x40075B),
        }

        self.cfg_fast_edges_check("x86_64", "cfg_switches", edges)

    def test_cfg_switches_armel(self):
        edges = {
            # jump table 0 in func_0
            (0x10434, 0x10488),
            (0x10434, 0x104E8),
            (0x10434, 0x10498),
            (0x10434, 0x104A8),
            (0x10434, 0x104B8),
            (0x10434, 0x104C8),
            (0x10434, 0x104D8),
            (0x10454, 0x104E8),  # default case
            # jump table 0 in func_1
            (0x10524, 0x105CC),
            (0x10524, 0x106B4),
            (0x10524, 0x105D8),
            (0x10524, 0x105E4),
            (0x10524, 0x105F0),
            (0x10524, 0x105FC),
            (0x10524, 0x10608),
            (0x10524, 0x10614),
            (0x10524, 0x10620),
            (0x10524, 0x1062C),
            (0x10524, 0x10638),
            (0x10534, 0x106B4),  # default case
            # jump table 1 in func_1
            (0x10650, 0x106A4),  # default case
            (0x10640, 0x10668),
            (0x10640, 0x10674),
            (0x10640, 0x10680),
            (0x10640, 0x1068C),
            (0x10640, 0x10698),
            # jump table 0 in main
            (0x10734, 0x107FC),
            (0x10734, 0x10808),
            (0x10734, 0x10818),
            (0x10734, 0x10828),
            (0x10734, 0x10838),
            (0x10734, 0x10848),
            (0x10734, 0x10858),
            (0x10734, 0x10864),
            (0x10744, 0x10864),  # default case
        }

        self.cfg_fast_edges_check("armel", "cfg_switches", edges)

    def test_cfg_switches_s390x(self):
        edges = {
            # jump table 0 in func_0
            (0x4007D4, 0x4007EA),  # case 1
            (0x4007D4, 0x4007F4),  # case 3
            (0x4007D4, 0x4007FE),  # case 5
            (0x4007D4, 0x400808),  # case 7
            (0x4007D4, 0x400812),  # case 9
            (0x4007D4, 0x40081C),  # case 12
            (0x4007C0, 0x4007CA),  # default case
            # jump table 0 in func_1
            (0x400872, 0x4008AE),  # case 2
            (0x400872, 0x4008BE),  # case 10
            (0x400872, 0x4008CE),  # case 12
            (0x400872, 0x4008DE),  # case 14
            (0x400872, 0x4008EE),  # case 15
            (0x400872, 0x4008FE),  # case 16
            (0x400872, 0x40090E),  # case 22
            (0x400872, 0x40091E),  # case 24
            (0x400872, 0x40092E),  # case 28
            (0x400872, 0x400888),  # case 38
            (0x400848, 0x400854),  # default case (1)
            (0x400872, 0x400854),  # default case (2)
            # jump table 1 in func_1
            (0x40093E, 0x400984),  # case 1
            (0x40093E, 0x400974),  # case 2
            (0x40093E, 0x400964),  # case 3
            (0x40093E, 0x400954),  # case 4
            (0x40093E, 0x400994),  # case 5
            (0x400898, 0x40089E),  # default case (1)
            # jump table 0 in main
            # case 1, 3, 5, 7, 9: optimized out
            (0x400638, 0x40064E),  # case 2
            (0x400638, 0x400692),  # case 4
            (0x400638, 0x4006A4),  # case 6
            (0x400638, 0x40066E),  # case 8
            (0x400638, 0x400680),  # case 10
            # case 45: optimized out
            (0x40062C, 0x40065C),  # default case
        }

        self.cfg_fast_edges_check("s390x", "cfg_switches", edges)

    def test_cfg_about_time(self):
        # This is to test the correctness of the PLT stub removal in CFGBase
        proj = angr.Project(os.path.join(test_location, "x86_64", "about_time"), auto_load_libs=False)
        cfg = proj.analyses.CFG()

        # a PLT stub that should be removed
        assert 0x401026 not in cfg.kb.functions
        # a PLT stub that should be removed
        assert 0x4010A6 not in cfg.kb.functions
        # a PLT stub that should be removed
        assert 0x40115E not in cfg.kb.functions
        # the start function that should not be removed
        assert proj.entry in cfg.kb.functions

    def test_segment_list_0(self):
        seg_list = SegmentList()
        seg_list.occupy(0, 1, "code")
        seg_list.occupy(2, 3, "code")

        assert len(seg_list) == 2
        assert seg_list._list[0].end == 1
        assert seg_list._list[1].end == 5
        assert seg_list.is_occupied(4)
        assert seg_list.is_occupied(5) is False

    def test_segment_list_1(self):
        seg_list = SegmentList()

        # They should be merged
        seg_list.occupy(0, 1, "code")
        seg_list.occupy(1, 2, "code")

        assert len(seg_list) == 1
        assert seg_list._list[0].start == 0
        assert seg_list._list[0].end == 3

    def test_segment_list_2(self):
        seg_list = SegmentList()

        # They should not be merged
        seg_list.occupy(0, 1, "code")
        seg_list.occupy(1, 2, "data")

        assert len(seg_list) == 2
        assert seg_list._list[0].start == 0
        assert seg_list._list[0].end == 1
        assert seg_list._list[1].start == 1
        assert seg_list._list[1].end == 3

    def test_segment_list_3(self):
        seg_list = SegmentList()

        # They should be merged, and create three different segments
        seg_list.occupy(0, 5, "code")
        seg_list.occupy(5, 5, "code")
        seg_list.occupy(1, 2, "data")

        assert len(seg_list) == 3

        assert seg_list._list[0].start == 0
        assert seg_list._list[0].end == 1
        assert seg_list._list[0].sort == "code"

        assert seg_list._list[1].start == 1
        assert seg_list._list[1].end == 3
        assert seg_list._list[1].sort == "data"

        assert seg_list._list[2].start == 3
        assert seg_list._list[2].end == 10
        assert seg_list._list[2].sort == "code"

    def test_segment_list_4(self):
        seg_list = SegmentList()

        seg_list.occupy(5, 5, "code")
        seg_list.occupy(4, 1, "code")
        seg_list.occupy(2, 2, "code")

        assert len(seg_list) == 1
        assert seg_list._list[0].start == 2
        assert seg_list._list[0].end == 10

    def test_segment_list_5(self):
        seg_list = SegmentList()

        seg_list.occupy(5, 5, "data")
        seg_list.occupy(4, 1, "code")
        seg_list.occupy(2, 2, "data")

        assert len(seg_list) == 3
        assert seg_list._list[0].start == 2
        assert seg_list._list[2].end == 10

        seg_list.occupy(3, 2, "data")

        assert len(seg_list) == 1
        assert seg_list._list[0].start == 2
        assert seg_list._list[0].end == 10

    def test_segment_list_6(self):
        seg_list = SegmentList()

        seg_list.occupy(10, 20, "code")
        seg_list.occupy(9, 2, "data")

        assert len(seg_list) == 2
        assert seg_list._list[0].start == 9
        assert seg_list._list[0].end == 11
        assert seg_list._list[0].sort == "data"

        assert seg_list._list[1].start == 11
        assert seg_list._list[1].end == 30
        assert seg_list._list[1].sort == "code"

    #
    # Serialization
    #

    def test_serialization_cfgnode(self):
        path = os.path.join(test_location, "x86_64", "fauxware")
        proj = angr.Project(path, auto_load_libs=False)

        cfg = proj.analyses.CFGFast()
        # the first node
        node = cfg.model.get_any_node(proj.entry)
        assert node is not None

        b = node.serialize()
        assert len(b) > 0
        new_node = CFGNode.parse(b)
        assert new_node.addr == node.addr
        assert new_node.size == node.size
        assert new_node.block_id == node.block_id

    def test_serialization_cfgfast(self):
        path = os.path.join(test_location, "x86_64", "fauxware")
        proj1 = angr.Project(path, auto_load_libs=False)
        proj2 = angr.Project(path, auto_load_libs=False)

        cfg = proj1.analyses.CFGFast()
        # parse the entire graph
        b = cfg.model.serialize()
        assert len(b) > 0

        # simulate importing a cfg from another tool
        cfg_model = CFGModel.parse(b, cfg_manager=proj2.kb.cfgs)

        assert len(cfg_model.graph.nodes) == len(cfg.graph.nodes)
        assert len(cfg_model.graph.edges) == len(cfg.graph.edges)

        n1 = cfg.model.get_any_node(proj1.entry)
        n2 = cfg_model.get_any_node(proj1.entry)
        assert n1 == n2

    #
    # CFG instance copy
    #

    def test_cfg_copy(self):
        path = os.path.join(test_location, "cgc", "CADET_00002")
        proj = angr.Project(path, auto_load_libs=False)

        cfg = proj.analyses.CFGFast()
        cfg_copy = cfg.copy()
        for attribute in cfg_copy.__dict__:
            if attribute in ["_graph", "_seg_list", "_model"]:
                continue
            assert getattr(cfg, attribute) == getattr(cfg_copy, attribute)

        assert id(cfg.model) != id(cfg_copy.model)
        assert id(cfg.model.graph) != id(cfg_copy.model.graph)
        assert id(cfg._seg_list) != id(cfg_copy._seg_list)

    #
    # Alignment bytes
    #

    def test_cfg_0_pe_msvc_debug_nocc(self):
        filename = os.path.join("windows", "msvc_cfg_0_debug.exe")
        proj = angr.Project(os.path.join(test_location, "x86_64", filename), auto_load_libs=False)
        cfg = proj.analyses.CFGFast()

        # make sure 0x140015683 is marked as alignments
        sort = cfg._seg_list.occupied_by_sort(0x140016583)
        assert sort == "alignment"

        assert 0x140015683 not in cfg.kb.functions

    #
    # Indirect jump resolvers
    #

    # For test cases for jump table resolver, please refer to test_jumptables.py

    def test_resolve_x86_elf_pic_plt(self):
        path = os.path.join(test_location, "i386", "fauxware_pie")
        proj = angr.Project(path, load_options={"auto_load_libs": False})

        cfg = proj.analyses.CFGFast()

        # puts
        puts_node = cfg.model.get_any_node(0x4005B0)
        assert puts_node is not None

        # there should be only one successor, which jumps to SimProcedure puts
        assert len(puts_node.successors) == 1
        puts_successor = puts_node.successors[0]
        assert puts_successor.addr == proj.loader.find_symbol("puts").rebased_addr

        # the SimProcedure puts should have more than one successors, which are all return targets
        assert len(puts_successor.successors) == 3
        simputs_successor = puts_successor.successors
        return_targets = {a.addr for a in simputs_successor}
        assert return_targets == {0x400800, 0x40087E, 0x4008B6}

    #
    # Function names
    #

    def test_function_names_for_unloaded_libraries(self):
        path = os.path.join(test_location, "i386", "fauxware_pie")
        proj = angr.Project(path, load_options={"auto_load_libs": False})

        cfg = proj.analyses.CFGFast()

        function_names = [f.name if not f.is_plt else "plt_" + f.name for f in cfg.functions.values()]

        assert "plt_puts" in function_names
        assert "plt_read" in function_names
        assert "plt___stack_chk_fail" in function_names
        assert "plt_exit" in function_names
        assert "puts" in function_names
        assert "read" in function_names
        assert "__stack_chk_fail" in function_names
        assert "exit" in function_names

    #
    # Basic blocks
    #

    def test_block_instruction_addresses_armhf(self):
        path = os.path.join(test_location, "armhf", "fauxware")
        proj = angr.Project(path, auto_load_libs=False)

        cfg = proj.analyses.CFGFast()

        main_func = cfg.kb.functions["main"]

        # all instruction addresses of the block must be odd
        block = next(b for b in main_func.blocks if b.addr == main_func.addr)

        assert len(block.instruction_addrs) == 12
        for instr_addr in block.instruction_addrs:
            assert instr_addr % 2 == 1

        main_node = cfg.model.get_any_node(main_func.addr)
        assert main_node is not None
        assert len(main_node.instruction_addrs) == 12
        for instr_addr in main_node.instruction_addrs:
            assert instr_addr % 2 == 1

    #
    # Tail-call optimization detection
    #

    def test_tail_call_optimization_detection_armel(self):
        # GitHub issue #1286

        path = os.path.join(test_location, "armel", "Nucleo_read_hyperterminal-stripped.elf")
        proj = angr.Project(path, auto_load_libs=False)

        cfg = proj.analyses.CFGFast(
            resolve_indirect_jumps=True,
            force_complete_scan=False,
            normalize=True,
            symbols=False,
            detect_tail_calls=True,
            data_references=True,
        )

        all_func_addrs = set(cfg.functions.keys())
        assert 0x80010B5 not in all_func_addrs
        assert 0x8003EF9 not in all_func_addrs
        assert 0x8008419 not in all_func_addrs

        # Functions that are jumped to from tail-calls
        tail_call_funcs = [
            0x8002BC1,
            0x80046C1,
            0x8000281,
            0x8001BDB,
            0x8002839,
            0x80037AD,
            0x8002C09,
            0x8004165,
            0x8004BE1,
            0x8002EB1,
        ]
        for member in tail_call_funcs:
            assert member in all_func_addrs

        # also test for tailcall return addresses

        # mapping of return blocks to return addrs that are the actual callers of certain tail-calls endpoints
        tail_call_return_addrs = {
            0x8002BD9: [0x800275F],  # 0x8002bc1
            0x80046D7: [0x800275F],  # 0x80046c1
            0x80046ED: [0x800275F],  # 0x80046c1
            0x8001BE7: [0x800068D, 0x8000695],  # 0x8001bdb ??
            0x800284D: [0x800028B, 0x80006E1, 0x80006E7],  # 0x8002839
            0x80037F5: [0x800270B, 0x8002733, 0x8002759, 0x800098F, 0x8000997],  # 0x80037ad
            0x80037EF: [0x800270B, 0x8002733, 0x8002759, 0x800098F, 0x8000997],  # 0x80037ad
            0x8002CC9: [
                0x8002D3B,
                0x8002B99,
                0x8002E9F,
                0x80041AD,
                0x8004C87,
                0x8004D35,
                0x8002EFB,
                0x8002BE9,
                0x80046EB,
                0x800464F,
                0x8002A09,
                0x800325F,
                0x80047C1,
            ],  # 0x8002c09
            0x8004183: [0x8002713],  # 0x8004165
            0x8004C31: [0x8002713],  # 0x8004be1
            0x8004C69: [0x8002713],  # 0x8004be1
            0x8002EF1: [0x800273B],
        }  # 0x8002eb1

        # check all expected return addrs are present
        for returning_block_addr, expected_return_addrs in tail_call_return_addrs.items():
            returning_block = cfg.model.get_any_node(returning_block_addr)
            return_block_addrs = [rb.addr for rb in cfg.model.get_successors(returning_block)]
            msg = "%x: unequal sizes of expected_addrs [%d] and return_block_addrs [%d]" % (
                returning_block_addr,
                len(expected_return_addrs),
                len(return_block_addrs),
            )
            assert len(return_block_addrs) == len(expected_return_addrs), msg
            for expected_addr in expected_return_addrs:
                msg = "expected retaddr {:x} not found for returning_block {:x}".format(
                    expected_addr,
                    returning_block_addr,
                )
                assert expected_addr in return_block_addrs, msg

    #
    # Incorrect function-leading blocks merging
    #

    def test_function_leading_blocks_merging(self):
        # GitHub issue #1312

        path = os.path.join(test_location, "armel", "Nucleo_read_hyperterminal-stripped.elf")
        proj = angr.Project(path, arch=archinfo.ArchARMCortexM(), auto_load_libs=False)

        cfg = proj.analyses.CFGFast(
            resolve_indirect_jumps=True,
            force_complete_scan=True,
            normalize=True,
            symbols=False,
            detect_tail_calls=True,
        )

        assert 0x8000799 in cfg.kb.functions
        assert 0x800079B not in cfg.kb.functions
        assert 0x800079B not in cfg.kb.functions[0x8000799].block_addrs_set
        assert 0x8000799 in cfg.kb.functions[0x8000799].block_addrs_set
        assert next(iter(b for b in cfg.kb.functions[0x8000799].blocks if b.addr == 0x8000799)).size == 6

    #
    # Blanket
    #

    def test_blanket_fauxware(self):
        path = os.path.join(test_location, "x86_64", "fauxware")
        proj = angr.Project(path, auto_load_libs=False)

        cfg = proj.analyses.CFGFast()

        cfb = proj.analyses.CFBlanket(kb=cfg.kb)

        # it should raise a key error when calling floor_addr on address 0 because nothing is mapped there
        # an instruction (or a block) starts at 0x400580
        assert cfb.floor_addr(0x400581) == 0x400580
        # a block ends at 0x4005a9 (exclusive)
        assert cfb.ceiling_addr(0x400581) == 0x4005A9

    #
    # CFG with patches
    #

    def test_unresolvable_targets(self):
        path = os.path.join(test_location, "cgc", "CADET_00002")
        proj = angr.Project(path, auto_load_libs=False)

        proj.analyses.CFGFast(normalize=True)
        func = proj.kb.functions[0x080489E0]

        true_endpoint_addrs = {0x8048BBC, 0x8048AF5, 0x8048B5C, 0x8048A41, 0x8048AA8}
        endpoint_addrs = {node.addr for node in func.endpoints}
        assert len(endpoint_addrs.symmetric_difference(true_endpoint_addrs)) == 0

    def test_indirect_jump_to_outside(self):
        # an indirect jump might be jumping to outside as well
        path = os.path.join(test_location, "mipsel", "libndpi.so.4.0.0")
        proj = angr.Project(path, auto_load_libs=False)

        cfg = proj.analyses.CFGFast()

        assert len(list(cfg.functions[0x404EE4].blocks)) == 3
        assert {ep.addr for ep in cfg.functions[0x404EE4].endpoints} == {
            0x404F00,
            0x404F08,
        }

    def test_plt_stub_has_one_jumpout_site(self):
        # each PLT stub must have exactly one jumpout site
        path = os.path.join(test_location, "x86_64", "1after909")
        proj = angr.Project(path, auto_load_libs=False)
        cfg = proj.analyses.CFGFast()

        for func in cfg.kb.functions.values():
            if func.is_plt:
                assert len(func.jumpout_sites) == 1

    def test_generate_special_info(self):
        path = os.path.join(test_location, "mipsel", "fauxware")
        proj = angr.Project(path, auto_load_libs=False)

        cfg = proj.analyses.CFGFast()

        assert any(func.info for func in cfg.functions.values())
        assert cfg.functions["main"].info["gp"] == 0x418CA0

    def test_load_from_shellcode(self):
        proj = angr.load_shellcode("loop: dec ecx; jnz loop; ret", "x86")
        cfg = proj.analyses.CFGFast()

        assert len(cfg.model.nodes()) == 2

    def test_starting_point_ordering(self):
        # project entry should always be first
        # so edge/path to unlabeled main function from _start
        # is correctly generated

        path = os.path.join(test_location, "armel", "start_ordering")
        proj = angr.Project(path, auto_load_libs=False)
        cfg = proj.analyses.CFGFast()

        # if ordering is incorrect, edge to function 0x103D4 will not exist
        n = cfg.model.get_any_node(proj.entry)
        assert n is not None
        assert len(n.successors) > 0
        assert len(n.successors[0].successors) > 0
        assert len(n.successors[0].successors[0].successors) == 3

        # now checking if path to the "real main" exists
        assert len(n.successors[0].successors[0].successors[1].successors) > 0
        n = n.successors[0].successors[0].successors[1].successors[0]

        assert len(n.successors) > 0
        assert len(n.successors[0].successors) > 0
        assert len(n.successors[0].successors[0].successors) > 0
        assert n.successors[0].successors[0].successors[0].addr == 0x103D4

    def test_error_returning(self):
        # error() is a great function: its returning depends on the value of the first argument...
        path = os.path.join(test_location, "x86_64", "mv_-O2")
        proj = angr.Project(path, auto_load_libs=False)
        cfg = proj.analyses.CFGFast()

        error_not_returning = [
            0x4030D4,
            0x403100,
            0x40313C,
            0x4031F5,
            0x40348A,
        ]

        error_returning = [0x403179, 0x4031A2, 0x403981, 0x403E30, 0x40403B]

        for error_site in error_not_returning:
            node = cfg.model.get_any_node(error_site)
            assert len(list(cfg.model.get_successors(node, excluding_fakeret=False))) == 1  # only the call successor

        for error_site in error_returning:
            node = cfg.model.get_any_node(error_site)
            assert len(list(cfg.model.get_successors(node, excluding_fakeret=False))) == 2  # both a call and a fakeret

    def test_kepler_server_armhf(self):
        binary_path = os.path.join(test_location, "armhf", "kepler_server")
        proj = angr.Project(binary_path, auto_load_libs=False)
        cfg = proj.analyses.CFG(
            normalize=True,
            indirect_calls_always_return=False,
        )

        func_main = cfg.kb.functions[0x10329]
        assert func_main.returning is False

        func_0 = cfg.kb.functions[0x15EE9]
        assert func_0.returning is False
        assert len(func_0.block_addrs_set) == 1

        func_1 = cfg.kb.functions[0x15D2D]
        assert func_1.returning is False

        func_2 = cfg.kb.functions[0x228C5]
        assert func_2.returning is False

        func_3 = cfg.kb.functions[0x12631]
        assert func_3.returning is True

    def test_func_in_added_segment_by_patcherex_arm(self):
        path = os.path.join(test_location, "armel", "patcherex", "replace_function_patch_with_function_reference")
        proj = angr.Project(path, auto_load_libs=False)
        cfg = proj.analyses.CFGFast(
            normalize=True,
            function_starts={0xA00081},
            regions=[
                (4195232, 4195244),
                (4195244, 4195324),
                (4195324, 4196016),
                (4196016, 4196024),
                (10485888, 10485950),
            ],
        )

        # Check whether the target function is in the functions list
        assert 0xA00081 in cfg.kb.functions
        # Check the number of basic blocks
        assert len(list(cfg.functions[0xA00081].blocks)) == 8

    def test_func_in_added_segment_by_patcherex_x64(self):
        path = os.path.join(test_location, "x86_64", "patchrex", "replace_function_patch_with_function_reference")
        proj = angr.Project(path, auto_load_libs=False)
        cfg = proj.analyses.CFGFast(
            normalize=True,
            function_starts={0xA0013D},
            regions=[
                (4195568, 4195591),
                (4195600, 4195632),
                (4195632, 4195640),
                (4195648, 4196418),
                (4196420, 4196429),
                (10486064, 10486213),
            ],
        )

        # Check whether the target function is in the functions list
        assert 0xA0013D in cfg.kb.functions
        # Check the number of basic blocks
        assert len(list(cfg.functions[0xA0013D].blocks)) == 7

    def test_indirect_calls_always_return_overly_aggressive(self):
        path = os.path.join(test_location, "x86_64", "ls_ubuntu_2004")
        proj = angr.Project(path, auto_load_libs=False)
        cfg = proj.analyses.CFGFast(normalize=True)
        node = cfg.model.get_any_node(0x404DB4)
        assert node is not None
        assert node.function_address == 0x40F770


class TestCfgfastDataReferences(unittest.TestCase):
    def test_data_references_x86_64(self):
        path = os.path.join(test_location, "x86_64", "fauxware")
        proj = angr.Project(path, auto_load_libs=False)

        cfg = proj.analyses.CFGFast(data_references=True)

        memory_data = cfg.memory_data
        # There is no code reference
        code_ref_count = len([d for d in memory_data.values() if d.sort == MemoryDataSort.CodeReference])
        assert code_ref_count >= 0, "There should be no code reference."

        # There are at least 2 pointer arrays
        ptr_array_count = len([d for d in memory_data.values() if d.sort == MemoryDataSort.PointerArray])
        assert ptr_array_count > 2, "Missing some pointer arrays."

        assert 0x4008D0 in memory_data
        sneaky_str = memory_data[0x4008D0]
        assert sneaky_str.sort == "string"
        assert sneaky_str.content == b"SOSNEAKY"

    def test_data_references_mipsel(self):
        path = os.path.join(test_location, "mipsel", "fauxware")
        proj = angr.Project(path, auto_load_libs=False)

        cfg = proj.analyses.CFGFast(data_references=True)

        memory_data = cfg.memory_data
        # There is no code reference
        code_ref_count = len([d for d in memory_data.values() if d.sort == MemoryDataSort.CodeReference])
        assert code_ref_count >= 0, "There should be no code reference."

        # There are at least 2 pointer arrays
        ptr_array_count = len([d for d in memory_data.values() if d.sort == MemoryDataSort.PointerArray])
        assert ptr_array_count >= 1, "Missing some pointer arrays."

        assert 0x400C00 in memory_data
        sneaky_str = memory_data[0x400C00]
        assert sneaky_str.sort == "string"
        assert sneaky_str.content == b"SOSNEAKY"

        assert 0x400C0C in memory_data
        str_ = memory_data[0x400C0C]
        assert str_.sort == "string"
        assert str_.content == b"Welcome to the admin console, trusted user!"

        assert 0x400C38 in memory_data
        str_ = memory_data[0x400C38]
        assert str_.sort == "string"
        assert str_.content == b"Go away!"

        assert 0x400C44 in memory_data
        str_ = memory_data[0x400C44]
        assert str_.sort == "string"
        assert str_.content == b"Username: "

        assert 0x400C50 in memory_data
        str_ = memory_data[0x400C50]
        assert str_.sort == "string"
        assert str_.content == b"Password: "

    def test_data_references_mips64(self):
        path = os.path.join(test_location, "mips64", "true")
        proj = angr.Project(path, auto_load_libs=False)

        cfg = proj.analyses.CFGFast(data_references=True, cross_references=True)
        memory_data = cfg.memory_data

        assert 0x120007DD8 in memory_data
        assert memory_data[0x120007DD8].sort == "string"
        assert memory_data[0x120007DD8].content == b"coreutils"

        xrefs = proj.kb.xrefs
        refs = list(xrefs.get_xrefs_by_dst(0x120007DD8))
        assert len(refs) == 2
        assert {x.ins_addr for x in refs} == {0x1200020E8, 0x120002108}

    def test_data_references_i386_gcc_pie(self):
        path = os.path.join(test_location, "i386", "nl")
        proj = angr.Project(path, auto_load_libs=False)

        cfg = proj.analyses.CFGFast(data_references=True, cross_references=True)
        memory_data = cfg.memory_data

        assert 0x405BB0 in memory_data
        assert memory_data[0x405BB0].sort == "string"
        assert memory_data[0x405BB0].content == b"/usr/local/share/locale"

        xrefs = proj.kb.xrefs
        refs = list(xrefs.get_xrefs_by_dst(0x405BB0))
        assert len(refs) == 1
        assert {x.ins_addr for x in refs} == {0x4011DD}

    def test_data_references_wide_string(self):
        path = os.path.join(test_location, "x86_64", "windows", "fauxware-wide.exe")
        proj = angr.Project(path, auto_load_libs=False)

        cfg = proj.analyses.CFGFast(data_references=True)
        recovered_strings = [d.content for d in cfg.memory_data.values() if d.sort == MemoryDataSort.UnicodeString]

        for testme in ("SOSNEAKY", "Welcome to the admin console, trusted user!\n", "Go away!\n", "Username: \n"):
            assert testme.encode("utf-16-le") in recovered_strings

    def test_arm_function_hints_from_data_references(self):
        path = os.path.join(test_location, "armel", "sha224sum")
        proj = angr.Project(path, auto_load_libs=False)

        proj.analyses.CFGFast(data_references=True)
        funcs = proj.kb.functions
        assert funcs.contains_addr(0x129C4)
        func = funcs[0x129C4]
        assert len(list(func.blocks)) == 1
        assert list(func.blocks)[0].size == 16


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
# pylint:disable=missing-class-docstring,no-self-use
__package__ = __package__ or "tests.analyses.decompiler"  # pylint:disable=redefined-builtin

import os

import unittest
import archinfo
import ailment
import angr
from angr.analyses.decompiler.peephole_optimizations import ConstantDereferences

from ...common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestPeepholeOptimizations(unittest.TestCase):
    def test_constant_dereference(self):
        # a = *(A) :=> a = the variable at at A iff
        # - A is a pointer that points to a read-only section.

        proj = angr.Project(os.path.join(test_location, "armel", "decompiler", "rm"), auto_load_libs=False)

        expr = ailment.Expr.Load(
            None,
            ailment.Expr.Const(None, None, 0xA000, proj.arch.bits),
            proj.arch.bytes,
            archinfo.Endness.LE,
            ins_addr=0x400100,
        )
        opt = ConstantDereferences(proj, proj.kb, 0)
        optimized = opt.optimize(expr)
        assert isinstance(optimized, ailment.Const)
        assert optimized.value == 0x183F8
        assert optimized.tags.get("ins_addr", None) == 0x400100, "Peephole optimizer lost tags."

        # multiple cases that no optimization should happen
        # a. Loading a pointer from a writable location
        expr = ailment.Expr.Load(None, ailment.Expr.Const(None, None, 0x21DF4, proj.arch.bits), 1, archinfo.Endness.LE)
        opt = ConstantDereferences(proj, proj.kb, 0)
        optimized = opt.optimize(expr)
        assert optimized is None


if __name__ == "__main__":
    unittest.main()



from ailment.expression import Convert, BinaryOp, Const, UnaryOp

from .base import PeepholeOptimizationExprBase


class SingleBitXor(PeepholeOptimizationExprBase):
    __slots__ = ()

    NAME = "bool_expr ^ 1 => !bool_expr (b)"
    expr_classes = (Convert,)

    def optimize(self, expr: Convert, **kwargs):
        # Convert(N->1, (Convert(1->N, t_x) ^ 0x1<N>) ==> Not(t_x)
        if expr.to_bits == 1:
            xor_expr = expr.operand
            if isinstance(xor_expr, BinaryOp) and xor_expr.op == "Xor":
                if (
                    isinstance(xor_expr.operands[0], Convert)
                    and isinstance(xor_expr.operands[1], Const)
                    and xor_expr.operands[1].value == 1
                    and xor_expr.operands[0].from_bits == 1
                ):
                    new_expr = UnaryOp(None, "Not", expr.operand.operands[0].operand)
                    return new_expr

        return expr



#!/usr/bin/env python3
__package__ = __package__ or "tests.exploration_techniques"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ..common import bin_location

test_location = os.path.join(bin_location, "tests")

find = {"veritesting_a": {"x86_64": 0x40066A}}

criteria = {"veritesting_a": lambda input_found: input_found.count(b"B") == 10}


class TestRun(unittest.TestCase):
    def _run_stochastic(self, binary, arch):
        proj = angr.Project(os.path.join(test_location, arch, binary), auto_load_libs=False)
        simgr = proj.factory.simulation_manager()
        start_state = simgr.active[0]
        technique = angr.exploration_techniques.StochasticSearch(start_state)
        simgr.use_technique(technique)

        def found(simgr):
            return simgr.active[0].addr == find[binary][arch]

        simgr.run(until=found)
        assert simgr.active[0].addr == find[binary][arch]

        input_found = simgr.active[0].posix.dumps(0)
        assert criteria[binary](input_found)

    def test_stochastic(self):
        self._run_stochastic("veritesting_a", "x86_64")


if __name__ == "__main__":
    unittest.main()



import angr


class IsProcessorFeaturePresent(angr.SimProcedure):
    def run(self, feature):  # pylint: disable=unused-argument,no-self-use,arguments-differ
        return 0  # we're dumb as shit!!!!



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("wer.dll")
prototypes = \
    {
        #
        'WerReportCreate': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="WER_REPORT_TYPE"), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "hProcess": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "wzConsentKey": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 64), "wzFriendlyEventName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 128), "wzApplicationName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 128), "wzApplicationPath": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260), "wzDescription": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 512), "hwndParent": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="WER_REPORT_INFORMATION", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzEventType", "repType", "pReportInformation", "phReportHandle"]),
        #
        'WerReportSetParameter': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hReportHandle", "dwparamID", "pwzName", "pwzValue"]),
        #
        'WerReportAddFile': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="WER_FILE_TYPE"), SimTypeInt(signed=False, label="WER_FILE")], SimTypeInt(signed=True, label="Int32"), arg_names=["hReportHandle", "pwzPath", "repFileType", "dwFileFlags"]),
        #
        'WerReportSetUIOption': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="WER_REPORT_UI"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hReportHandle", "repUITypeID", "pwzValue"]),
        #
        'WerReportSubmit': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="WER_CONSENT"), SimTypeInt(signed=False, label="WER_SUBMIT_FLAGS"), SimTypePointer(SimTypeInt(signed=False, label="WER_SUBMIT_RESULT"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hReportHandle", "consent", "dwFlags", "pSubmitResult"]),
        #
        'WerReportAddDump': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="WER_DUMP_TYPE"), SimTypePointer(SimStruct({"pExceptionPointers": SimTypePointer(SimStruct({"ExceptionRecord": SimTypePointer(SimStruct({"ExceptionCode": SimTypeInt(signed=True, label="Int32"), "ExceptionFlags": SimTypeInt(signed=False, label="UInt32"), "ExceptionRecord": SimTypePointer(SimTypeBottom(label="EXCEPTION_RECORD"), offset=0), "ExceptionAddress": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "NumberParameters": SimTypeInt(signed=False, label="UInt32"), "ExceptionInformation": SimTypeFixedSizeArray(SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0), 15)}, name="EXCEPTION_RECORD", pack=False, align=None), offset=0), "ContextRecord": SimTypePointer(SimStruct({"P1Home": SimTypeLongLong(signed=False, label="UInt64"), "P2Home": SimTypeLongLong(signed=False, label="UInt64"), "P3Home": SimTypeLongLong(signed=False, label="UInt64"), "P4Home": SimTypeLongLong(signed=False, label="UInt64"), "P5Home": SimTypeLongLong(signed=False, label="UInt64"), "P6Home": SimTypeLongLong(signed=False, label="UInt64"), "ContextFlags": SimTypeInt(signed=False, label="UInt32"), "MxCsr": SimTypeInt(signed=False, label="UInt32"), "SegCs": SimTypeShort(signed=False, label="UInt16"), "SegDs": SimTypeShort(signed=False, label="UInt16"), "SegEs": SimTypeShort(signed=False, label="UInt16"), "SegFs": SimTypeShort(signed=False, label="UInt16"), "SegGs": SimTypeShort(signed=False, label="UInt16"), "SegSs": SimTypeShort(signed=False, label="UInt16"), "EFlags": SimTypeInt(signed=False, label="UInt32"), "Dr0": SimTypeLongLong(signed=False, label="UInt64"), "Dr1": SimTypeLongLong(signed=False, label="UInt64"), "Dr2": SimTypeLongLong(signed=False, label="UInt64"), "Dr3": SimTypeLongLong(signed=False, label="UInt64"), "Dr6": SimTypeLongLong(signed=False, label="UInt64"), "Dr7": SimTypeLongLong(signed=False, label="UInt64"), "Rax": SimTypeLongLong(signed=False, label="UInt64"), "Rcx": SimTypeLongLong(signed=False, label="UInt64"), "Rdx": SimTypeLongLong(signed=False, label="UInt64"), "Rbx": SimTypeLongLong(signed=False, label="UInt64"), "Rsp": SimTypeLongLong(signed=False, label="UInt64"), "Rbp": SimTypeLongLong(signed=False, label="UInt64"), "Rsi": SimTypeLongLong(signed=False, label="UInt64"), "Rdi": SimTypeLongLong(signed=False, label="UInt64"), "R8": SimTypeLongLong(signed=False, label="UInt64"), "R9": SimTypeLongLong(signed=False, label="UInt64"), "R10": SimTypeLongLong(signed=False, label="UInt64"), "R11": SimTypeLongLong(signed=False, label="UInt64"), "R12": SimTypeLongLong(signed=False, label="UInt64"), "R13": SimTypeLongLong(signed=False, label="UInt64"), "R14": SimTypeLongLong(signed=False, label="UInt64"), "R15": SimTypeLongLong(signed=False, label="UInt64"), "Rip": SimTypeLongLong(signed=False, label="UInt64"), "Anonymous": SimUnion({"FltSave": SimTypeBottom(label="XSAVE_FORMAT"), "Anonymous": SimStruct({"Header": SimTypeFixedSizeArray(SimTypeBottom(label="M128A"), 2), "Legacy": SimTypeFixedSizeArray(SimTypeBottom(label="M128A"), 8), "Xmm0": SimTypeBottom(label="M128A"), "Xmm1": SimTypeBottom(label="M128A"), "Xmm2": SimTypeBottom(label="M128A"), "Xmm3": SimTypeBottom(label="M128A"), "Xmm4": SimTypeBottom(label="M128A"), "Xmm5": SimTypeBottom(label="M128A"), "Xmm6": SimTypeBottom(label="M128A"), "Xmm7": SimTypeBottom(label="M128A"), "Xmm8": SimTypeBottom(label="M128A"), "Xmm9": SimTypeBottom(label="M128A"), "Xmm10": SimTypeBottom(label="M128A"), "Xmm11": SimTypeBottom(label="M128A"), "Xmm12": SimTypeBottom(label="M128A"), "Xmm13": SimTypeBottom(label="M128A"), "Xmm14": SimTypeBottom(label="M128A"), "Xmm15": SimTypeBottom(label="M128A")}, name="_Anonymous_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), "VectorRegister": SimTypeFixedSizeArray(SimTypeBottom(label="M128A"), 26), "VectorControl": SimTypeLongLong(signed=False, label="UInt64"), "DebugControl": SimTypeLongLong(signed=False, label="UInt64"), "LastBranchToRip": SimTypeLongLong(signed=False, label="UInt64"), "LastBranchFromRip": SimTypeLongLong(signed=False, label="UInt64"), "LastExceptionToRip": SimTypeLongLong(signed=False, label="UInt64"), "LastExceptionFromRip": SimTypeLongLong(signed=False, label="UInt64")}, name="CONTEXT", pack=False, align=None), offset=0)}, name="EXCEPTION_POINTERS", pack=False, align=None), offset=0), "bClientPointers": SimTypeInt(signed=True, label="Int32")}, name="WER_EXCEPTION_INFORMATION", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"dwSize": SimTypeInt(signed=False, label="UInt32"), "dwMask": SimTypeInt(signed=False, label="UInt32"), "dwDumpFlags": SimTypeInt(signed=False, label="UInt32"), "bOnlyThisThread": SimTypeInt(signed=True, label="Int32"), "dwExceptionThreadFlags": SimTypeInt(signed=False, label="UInt32"), "dwOtherThreadFlags": SimTypeInt(signed=False, label="UInt32"), "dwExceptionThreadExFlags": SimTypeInt(signed=False, label="UInt32"), "dwOtherThreadExFlags": SimTypeInt(signed=False, label="UInt32"), "dwPreferredModuleFlags": SimTypeInt(signed=False, label="UInt32"), "dwOtherModuleFlags": SimTypeInt(signed=False, label="UInt32"), "wzPreferredModuleList": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 256)}, name="WER_DUMP_CUSTOM_OPTIONS", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["hReportHandle", "hProcess", "hThread", "dumpType", "pExceptionParam", "pDumpCustomOptions", "dwFlags"]),
        #
        'WerReportCloseHandle': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hReportHandle"]),
        #
        'WerAddExcludedApplication': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzExeName", "bAllUsers"]),
        #
        'WerRemoveExcludedApplication': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzExeName", "bAllUsers"]),
        #
        'WerStoreOpen': SimTypeFunction([SimTypeInt(signed=False, label="REPORT_STORE_TYPES"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["repStoreType", "phReportStore"]),
        #
        'WerStoreClose': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["hReportStore"]),
        #
        'WerStoreGetFirstReportKey': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hReportStore", "ppszReportKey"]),
        #
        'WerStoreGetNextReportKey': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hReportStore", "ppszReportKey"]),
        #
        'WerStoreQueryReportMetadataV2': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"Signature": SimStruct({"EventName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 65), "Parameters": SimTypeFixedSizeArray(SimStruct({"Name": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "Value": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260)}, name="WER_REPORT_PARAMETER", pack=False, align=None), 10)}, name="WER_REPORT_SIGNATURE", pack=False, align=None), "BucketId": SimTypeBottom(label="Guid"), "ReportId": SimTypeBottom(label="Guid"), "CreationTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "SizeInBytes": SimTypeLongLong(signed=False, label="UInt64"), "CabId": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260), "ReportStatus": SimTypeInt(signed=False, label="UInt32"), "ReportIntegratorId": SimTypeBottom(label="Guid"), "NumberOfFiles": SimTypeInt(signed=False, label="UInt32"), "SizeOfFileNames": SimTypeInt(signed=False, label="UInt32"), "FileNames": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="WER_REPORT_METADATA_V2", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hReportStore", "pszReportKey", "pReportMetadata"]),
        #
        'WerStoreQueryReportMetadataV3': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"Signature": SimStruct({"EventName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 65), "Parameters": SimTypeFixedSizeArray(SimStruct({"Name": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "Value": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260)}, name="WER_REPORT_PARAMETER", pack=False, align=None), 10)}, name="WER_REPORT_SIGNATURE", pack=False, align=None), "BucketId": SimTypeBottom(label="Guid"), "ReportId": SimTypeBottom(label="Guid"), "CreationTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "SizeInBytes": SimTypeLongLong(signed=False, label="UInt64"), "CabId": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260), "ReportStatus": SimTypeInt(signed=False, label="UInt32"), "ReportIntegratorId": SimTypeBottom(label="Guid"), "NumberOfFiles": SimTypeInt(signed=False, label="UInt32"), "SizeOfFileNames": SimTypeInt(signed=False, label="UInt32"), "FileNames": SimTypePointer(SimTypeChar(label="Char"), offset=0), "FriendlyEventName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 128), "ApplicationName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 128), "ApplicationPath": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260), "Description": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 512), "BucketIdString": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260), "LegacyBucketId": SimTypeLongLong(signed=False, label="UInt64")}, name="WER_REPORT_METADATA_V3", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hReportStore", "pszReportKey", "pReportMetadata"]),
        #
        'WerFreeString': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeBottom(label="Void"), arg_names=["pwszStr"]),
        #
        'WerStorePurge': SimTypeFunction([], SimTypeInt(signed=True, label="Int32")),
        #
        'WerStoreGetReportCount': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hReportStore", "pdwReportCount"]),
        #
        'WerStoreGetSizeOnDisk': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeLongLong(signed=False, label="UInt64"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hReportStore", "pqwSizeInBytes"]),
        #
        'WerStoreQueryReportMetadataV1': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"Signature": SimStruct({"EventName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 65), "Parameters": SimTypeFixedSizeArray(SimStruct({"Name": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 129), "Value": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 260)}, name="WER_REPORT_PARAMETER", pack=False, align=None), 10)}, name="WER_REPORT_SIGNATURE", pack=False, align=None), "BucketId": SimTypeBottom(label="Guid"), "ReportId": SimTypeBottom(label="Guid"), "CreationTime": SimStruct({"dwLowDateTime": SimTypeInt(signed=False, label="UInt32"), "dwHighDateTime": SimTypeInt(signed=False, label="UInt32")}, name="FILETIME", pack=False, align=None), "SizeInBytes": SimTypeLongLong(signed=False, label="UInt64")}, name="WER_REPORT_METADATA_V1", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hReportStore", "pszReportKey", "pReportMetadata"]),
        #
        'WerStoreUploadReport': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=False, label="WER_SUBMIT_RESULT"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["hReportStore", "pszReportKey", "dwFlags", "pSubmitResult"]),
    }

lib.set_prototypes(prototypes)



from typing import Optional, List, Dict

from cle.address_translator import AddressTranslator
from sortedcontainers import SortedDict

from .plugin import KnowledgeBasePlugin


# TODO: Serializable
class Patch:
    def __init__(self, addr, new_bytes, comment: Optional[str] = None):
        self.addr = addr
        self.new_bytes = new_bytes
        self.comment = comment

    def __len__(self):
        return len(self.new_bytes)


class PatchManager(KnowledgeBasePlugin):
    """
    A placeholder-style implementation for a binary patch manager. This class should be significantly changed in the
    future when all data about loaded binary objects are loaded into angr knowledge base from CLE. As of now, it only
    stores byte-level replacements.

    Patches should not overlap, but it's user's responsibility to check for and avoid overlapping patches.
    """

    def __init__(self, kb):
        super().__init__()

        self._patches: Dict[int, Patch] = SortedDict()
        self._kb = kb
        self._patched_entry_state = None

    def add_patch(self, addr, new_bytes, comment: Optional[str] = None):
        self._patches[addr] = Patch(addr, new_bytes, comment=comment)
        self._patched_entry_state = None

    def add_patch_obj(self, patch: Patch):
        self._patches[patch.addr] = patch
        self._patched_entry_state = None

    def remove_patch(self, addr):
        if addr in self._patches:
            del self._patches[addr]
        self._patched_entry_state = None

    def patch_addrs(self):
        return self._patches.keys()

    def get_patch(self, addr):
        """
        Get patch at the given address.

        :param int addr:    The address of the patch.
        :return:            The patch if there is one starting at the address, or None if there isn't any.
        :rtype:             Patch or None
        """
        return self._patches.get(addr, None)

    def get_all_patches(self, addr, size):
        """
        Retrieve all patches that cover a region specified by [addr, addr+size).

        :param int addr:    The address of the beginning of the region.
        :param int size:    Size of the region.
        :return:            A list of patches.
        :rtype:             list
        """
        patches = []
        for patch_addr in self._patches.irange(maximum=addr + size - 1, reverse=True):
            p = self._patches[patch_addr]
            if self.overlap(p.addr, p.addr + len(p), addr, addr + size):
                patches.append(p)
            else:
                break
        return patches[::-1]

    def keys(self):
        return self._patches.keys()

    def items(self):
        return self._patches.items()

    def values(self):
        return self._patches.values()

    def copy(self):
        o = PatchManager(self._kb)
        o._patches = self._patches.copy()

    @staticmethod
    def overlap(a0, a1, b0, b1):
        return a0 <= b0 < a1 or a0 <= b1 < a1 or b0 <= a0 < b1

    def apply_patches_to_binary(
        self, binary_bytes: Optional[bytes] = None, patches: Optional[List[Patch]] = None
    ) -> bytes:
        if patches is None:
            patches = sorted(list(self._patches.values()), key=lambda x: x.addr)

        if binary_bytes is None:
            with open(self._kb._project.loader.main_object.binary, "rb") as f:
                binary_bytes = f.read()
        for patch in patches:
            # convert addr to file offset
            at = AddressTranslator.from_mva(patch.addr, self._kb._project.loader.main_object)
            file_offset = at.to_raw()

            if file_offset < len(binary_bytes) and file_offset + len(patch.new_bytes) < len(binary_bytes):
                binary_bytes = (
                    binary_bytes[:file_offset] + patch.new_bytes + binary_bytes[file_offset + len(patch.new_bytes) :]
                )

        return binary_bytes

    def apply_patches_to_state(self, state):
        for patch in self._patches.values():
            state.memory.store(patch.addr, patch.new_bytes)

    @property
    def patched_entry_state(self):
        if self._patched_entry_state is None:
            self._patched_entry_state = self._kb._project.factory.entry_state()
            self.apply_patches_to_state(self._patched_entry_state)
        return self._patched_entry_state


KnowledgeBasePlugin.register_default("patches", PatchManager)



#!/usr/bin/env python3
from unittest import mock
import unittest

import networkx

import ailment

from archinfo.arch_x86 import ArchX86
from angr.analyses.forward_analysis.visitors import FunctionGraphVisitor
from angr.analyses.reaching_definitions.subject import Subject, SubjectType
from angr.block import Block
from angr.knowledge_plugins import Function


def _a_mock_function(address, name):
    return Function(None, address, name=name, syscall=False, is_simprocedure=False, is_plt=False, returning=False)


class TestSubject(unittest.TestCase):
    @mock.patch.object(Function, "_get_initial_binary_name", return_value="binary")
    def test_can_be_instantiated_with_a_function(self, _):
        function = _a_mock_function(0x42, "function_name")
        subject = Subject(function)

        assert subject.content == function
        assert subject.type == SubjectType.Function

    @mock.patch.object(Block, "_parse_vex_info", return_value=None)
    def test_can_be_instantiated_with_a_block(self, _):
        arch = ArchX86()
        block = Block(0x42, byte_string=b"", arch=arch)
        subject = Subject(block)

        assert subject.content == block
        assert subject.type == SubjectType.Block

    def test_can_be_instantiated_with_an_ailment_block(self):
        block = ailment.Block(0x42, original_size=4)
        subject = Subject(block)

        assert subject.content == block
        assert subject.type == SubjectType.Block

    def test_fails_when_instanciated_with_an_inadequate_object(self):
        self.assertRaises(TypeError, Subject, "test-me", None)

    @mock.patch.object(Function, "_get_initial_binary_name", return_value="binary")
    @mock.patch.object(FunctionGraphVisitor, "sort_nodes")
    def test_when_instanciated_with_a_function_need_other_attributes(self, _, __):
        function = _a_mock_function(0x42, "function_name")
        func_graph = networkx.DiGraph()
        cc = "mock_cc"

        subject = Subject(function, func_graph, cc)

        assert subject.func_graph == func_graph
        assert subject.cc == cc

    def test_cc_attribute_should_raise_error_when_subject_is_a_block(self):
        arch = ArchX86()
        block = Block(0x42, byte_string=b"", arch=arch)
        subject = Subject(block)
        with self.assertRaises(TypeError):
            _ = subject.cc

    def test_func_graph_attribute_should_raise_error_when_subject_is_a_block(self):
        arch = ArchX86()
        block = Block(0x42, byte_string=b"", arch=arch)
        subject = Subject(block)
        with self.assertRaises(TypeError):
            _ = subject.func_graph


if __name__ == "__main__":
    unittest.main()



import angr


class getenv(angr.SimProcedure):
    """
    The getenv() function searches the environment list to find the
    environment variable name, and returns a pointer to the
    corresponding value string.
    """

    # pylint:disable=arguments-differ

    def run(self, m_name):
        strlen = angr.SIM_PROCEDURES["libc"]["strlen"]
        name_len = self.inline_call(strlen, m_name)
        name_expr = self.state.memory.load(m_name, name_len.max_null_index, endness="Iend_BE")
        name = self.state.solver.eval(name_expr, cast_to=bytes)

        p = self.state.posix.environ
        if p is None:
            return self.state.solver.BVS(b"getenv__" + name, self.state.arch.bits, key=("api", "getenv", name.decode()))

        while True:
            m_line = self.state.memory.load(p, self.state.arch.bytes, endness=self.arch.memory_endness)
            if self.state.solver.eval(m_line, cast_to=int) == 0:
                break
            line_len = self.inline_call(strlen, m_line)
            line_expr = self.state.memory.load(m_line, line_len.max_null_index, endness="Iend_BE")
            line = self.state.solver.eval(line_expr, cast_to=bytes)
            kv = line.split(b"=", maxsplit=1)
            if len(kv) == 2 and kv[0] == name:
                return m_line + (name_len.max_null_index + 1)
            p += self.state.arch.bytes
        return 0



from ailment.expression import BinaryOp, Const, Convert

from .base import PeepholeOptimizationExprBase


class ASubADivConstMulConst(PeepholeOptimizationExprBase):
    __slots__ = ()

    NAME = "a - (a / N) * N => a % N"
    expr_classes = (BinaryOp,)

    def optimize(self, expr: BinaryOp, **kwargs):
        if (
            expr.op == "Sub"
            and len(expr.operands) == 2
            and isinstance(expr.operands[1], BinaryOp)
            and expr.operands[1].op == "Mul"
            and isinstance(expr.operands[1].operands[1], Const)
        ):
            a0, op1 = expr.operands
            op1_left = op1.operands[0]
            mul_const = expr.operands[1].operands[1]

            if (
                isinstance(op1_left, Convert)
                and isinstance(a0, Convert)
                and op1_left.to_bits == a0.to_bits
                and op1_left.from_bits == a0.from_bits
            ):
                # Convert(a) - (Convert(a / N)) * N  ==>  Convert(a) % N
                conv_expr = a0
                a0 = a0.operand
                op1_left = op1_left.operand
            else:
                conv_expr = None

            if isinstance(op1_left, BinaryOp) and op1_left.op == "Div" and isinstance(op1_left.operands[1], Const):
                # a - (a / N) * N  ==>  a % N
                a1 = op1_left.operands[0]
                div_const = op1_left.operands[1]

                if a0.likes(a1) and mul_const.value == div_const.value:
                    operands = [a0, div_const]
                    mod = BinaryOp(expr.idx, "Mod", operands, False, bits=a0.bits, **expr.tags)
                    if conv_expr is not None:
                        mod = Convert(
                            conv_expr.idx,
                            conv_expr.from_bits,
                            conv_expr.to_bits,
                            conv_expr.is_signed,
                            mod,
                            **conv_expr.tags,
                        )
                    return mod

        return None



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.exploration_techniques"  # pylint:disable=redefined-builtin

import unittest
import os

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


def _ultra_oppologist(p, s):
    old_ops = dict(angr.engines.vex.claripy.irop.operations)
    try:
        angr.engines.vex.claripy.irop.operations.clear()
        angr.engines.vex.claripy.irop.operations["Iop_Add32"] = old_ops["Iop_Add32"]

        pg = p.factory.simulation_manager(s)
        pg.use_technique(angr.exploration_techniques.Oppologist())
        pg.explore()

        return pg
    finally:
        angr.engines.vex.claripy.irop.operations.update(old_ops)


class TestOppologist(unittest.TestCase):
    def test_fauxware_oppologist(self):
        p = angr.Project(os.path.join(test_location, "i386", "fauxware"))
        s = p.factory.full_init_state(remove_options={angr.options.LAZY_SOLVES, angr.options.EXTENDED_IROP_SUPPORT})

        pg = _ultra_oppologist(p, s)
        assert len(pg.deadended) == 1
        assert len(pg.deadended[0].posix.dumps(0)) == 18
        stdout = pg.deadended[0].posix.dumps(1)
        if b"trusted user" in stdout:
            assert stdout.count(b"\n") == 3
        else:
            assert stdout.count(b"\n") == 2

    def test_cromu_70(self):
        p = angr.Project(os.path.join(test_location, "cgc", "CROMU_00070"))
        inp = bytes.fromhex(
            "030e000001000001001200010000586d616ce000000600030000040dd0000000000600000606000006030e000001000001003200010000586d616ce0030000000000030e000001000001003200010000586d616ce003000000000006000006030e000001000001003200010000586d616ce0030000df020000"
        )
        s = p.factory.full_init_state(
            add_options={angr.options.UNICORN},
            remove_options={angr.options.LAZY_SOLVES, angr.options.SUPPORT_FLOATING_POINT},
            stdin=inp,
        )

        pg = p.factory.simulation_manager(s)
        pg.use_technique(angr.exploration_techniques.Oppologist())
        pg.run(n=50)
        assert pg.one_active.history.block_count > 1500


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
__package__ = __package__ or "tests.factory.block"  # pylint:disable=redefined-builtin

import logging
import unittest
import os

import angr

from ...common import bin_location


test_location = os.path.join(bin_location, "tests")
l = logging.getLogger("angr.tests")


# pylint: disable=missing-class-docstring
# pylint: disable=no-self-use
class TestBlockCache(unittest.TestCase):
    def test_block_cache(self):
        p = angr.Project(
            os.path.join(test_location, "x86_64", "fauxware"), translation_cache=True, auto_load_libs=False
        )
        b = p.factory.block(p.entry)
        assert p.factory.block(p.entry).vex is b.vex

        p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), translation_cache=False)
        b = p.factory.block(p.entry)
        assert p.factory.block(p.entry).vex is not b.vex


if __name__ == "__main__":
    unittest.main()



import angr
import logging

l = logging.getLogger(name=__name__)


class getcwd(angr.SimProcedure):
    def run(self, buf, size):
        if self.state.solver.unique(size):
            size = self.state.solver.eval_one(size)

        cwd = self.state.fs.cwd + b"\0"
        if len(cwd) > size:
            return -self.state.posix.ERANGE
        try:
            self.state.memory.store(buf, cwd, size=len(cwd))
        except angr.errors.SimSegfaultException:
            return -self.state.posix.EFAULT
        return len(cwd)


class chdir(angr.SimProcedure):
    def run(self, buf):
        cwd = self.state.mem[buf].string.concrete
        l.info("chdir(%r)", cwd)
        self.state.fs.cwd = cwd
        return 0



from ailment.expression import Expression, Tmp
from ailment import AILBlockWalkerBase


class TmpvarFinder(AILBlockWalkerBase):
    """
    Walks an AIL expression to find Tmp expressions.
    """

    def __init__(self, expr: Expression):
        super().__init__()
        self.has_tmp = False

        self.walk_expression(expr)

    def _handle_Tmp(self, expr_idx: int, expr: Tmp, stmt_idx: int, stmt, block):
        self.has_tmp = True



#!/usr/bin/env python3
__package__ = __package__ or "tests.exploration_techniques"  # pylint:disable=redefined-builtin

import tempfile
import os
import logging
import unittest

import angr

from ..common import bin_location, broken


test_location = os.path.join(bin_location, "tests")
l = logging.getLogger("angr_tests.managers")


# pylint: disable=missing-class-docstring
# pylint: disable=no-self-use
class TestCacher(unittest.TestCase):
    @broken
    def test_cacher(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), load_options={"auto_load_libs": False})

        tmp_dir = tempfile.mkdtemp(prefix="test_cacher_container")
        container = os.path.join(tmp_dir, "%s.cache" % os.path.basename(p.filename))

        pg = p.factory.simulation_manager()
        pg.use_technique(angr.exploration_techniques.Cacher(when=0x4006EE, container=container))
        pg.run()

        pg2 = p.factory.simulation_manager()
        pg2.use_technique(angr.exploration_techniques.Cacher(container=container))
        assert pg2.active[0].addr == 0x4006ED

        pg2.run()

        assert len(pg2.deadended) == len(pg.deadended)
        assert pg2.deadended[0].addr in [s.addr for s in pg.deadended]
        assert pg2.deadended[1].addr in [s.addr for s in pg.deadended]
        assert pg2.deadended[2].addr in [s.addr for s in pg.deadended]


if __name__ == "__main__":
    unittest.main()



import logging

from archinfo.arch_soot import SootAddressDescriptor

from ..exceptions import IncorrectLocationException
from ..expressions import translate_expr
from ..values import translate_value

l = logging.getLogger("angr.engines.soot.statements.if")


class SimSootStmt:
    """
    The base class of all Soot statements.
    """

    def __init__(self, stmt, state):
        self.stmt = stmt
        self.state = state

        self.invoke_expr = None
        self.jmp_targets_with_conditions = []

    def process(self):
        """
        Process the statement and apply all effects on the state.
        :return: None
        """
        self._execute()

    def _execute(self):
        raise NotImplementedError()

    def _translate_expr(self, expr):
        expr_ = translate_expr(expr, self.state)
        return expr_

    def _translate_value(self, value):
        value_ = translate_value(value, self.state)
        return value_

    def _get_bb_addr_from_instr(self, instr):
        """
        Returns the address of the methods basic block that contains the given
        instruction.

        :param instr: The index of the instruction (within the current method).
        :rtype: SootAddressDescriptor
        """
        current_method = self.state.addr.method
        try:
            bb = current_method.block_by_label[instr]
        except KeyError:
            l.error("Possible jump to a non-existing bb %s --> %d", self.state.addr, instr)
            raise IncorrectLocationException()

        return SootAddressDescriptor(current_method, bb.idx, 0)

    #
    # Jumps
    #

    def _add_jmp_target(self, target, condition):
        self.jmp_targets_with_conditions += [(target, condition)]

    @property
    def has_jump_targets(self):
        return self.jmp_targets_with_conditions != []

    #
    # Invocations
    #

    def _add_invoke_target(self, invoke_expr, ret_var=None):
        self.invoke_expr = invoke_expr
        self.invoke_expr.ret_var = ret_var

    @property
    def has_invoke_target(self):
        return self.invoke_expr is not None



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,disable=no-self-use
__package__ = __package__ or "tests.analyses"  # pylint:disable=redefined-builtin

import os
import unittest

import angr

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestInitFinder(unittest.TestCase):
    def test_p2im_drone(self):
        bin_path = os.path.join(test_location, "armel", "p2im_drone.elf")
        proj = angr.Project(bin_path, auto_load_libs=False)
        cfg = proj.analyses.CFG(data_references=True)

        func = cfg.functions["Peripherals_Init"]
        state = proj.factory.blank_state()
        prop = proj.analyses.Propagator(func=func, base_state=state)

        init_finder = proj.analyses.InitializationFinder(func=func, replacements=prop.replacements)
        overlay = init_finder.overlay

        # h12c1.Instance
        assert state.solver.eval_one(overlay.load(0x20001500, 4, endness="Iend_LE")) == 0x40005400
        # hi2c1.Init.AddressingMode
        assert state.solver.eval_one(overlay.load(0x20001500 + 4 + 0xC, 4, endness="Iend_LE")) == 0x4000
        # h12c1.Init.NoStretchMode
        assert state.solver.eval_one(overlay.load(0x20001500 + 4 + 0x1C, 4, endness="Iend_LE")) == 0


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
__package__ = __package__ or "tests.knowledge_plugins.functions"  # pylint:disable=redefined-builtin

import os
import unittest

from archinfo import ArchAMD64

import angr
from angr.utils.constants import DEFAULT_STATEMENT

from ...common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestFunctionManager(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.project = angr.Project(os.path.join(test_location, "x86_64", "fauxware"), auto_load_libs=False)

    def test_amd64(self):
        expected_functions = {
            0x4004E0,
            0x400510,
            0x400520,
            0x400530,
            0x400540,
            0x400550,
            0x400560,
            0x400570,
            0x400580,
            0x4005AC,
            0x400640,
            0x400664,
            0x4006ED,
            0x4006FD,
            0x40071D,
            0x4007E0,
            0x400880,
        }
        expected_blocks = {
            0x40071D,
            0x40073E,
            0x400754,
            0x40076A,
            0x400774,
            0x40078A,
            0x4007A0,
            0x4007B3,
            0x4007C7,
            0x4007C9,
            0x4007BD,
            0x4007D3,
        }
        expected_callsites = {0x40071D, 0x40073E, 0x400754, 0x40076A, 0x400774, 0x40078A, 0x4007A0, 0x4007BD, 0x4007C9}
        expected_callsite_targets = {4195600, 4195632, 4195632, 4195600, 4195632, 4195632, 4195940, 4196077, 4196093}
        expected_callsite_returns = {
            0x40073E,
            0x400754,
            0x40076A,
            0x400774,
            0x40078A,
            0x4007A0,
            0x4007B3,
            0x4007C7,
            None,
        }

        self.project.analyses.CFGEmulated()
        assert {k for k in self.project.kb.functions.keys() if k < 0x500000} == expected_functions

        main = self.project.kb.functions.function(name="main")
        assert main.startpoint.addr == 0x40071D
        assert set(main.block_addrs) == expected_blocks
        assert [0x4007D3] == [bl.addr for bl in main.endpoints]
        assert set(main.get_call_sites()) == expected_callsites
        assert set(map(main.get_call_target, main.get_call_sites())) == expected_callsite_targets
        assert set(map(main.get_call_return, main.get_call_sites())) == expected_callsite_returns
        assert main.has_return

        rejected = self.project.kb.functions.function(name="rejected")
        assert rejected.returning is False

        # transition graph
        main_g = main.transition_graph
        main_g_edges_ = main_g.edges(data=True)

        # Convert nodes those edges from blocks to addresses
        main_g_edges = []
        for src_node, dst_node, data in main_g_edges_:
            main_g_edges.append((src_node.addr, dst_node.addr, data))

        edges = [
            (0x40071D, 0x400510, {"type": "call", "stmt_idx": DEFAULT_STATEMENT, "ins_addr": 0x400739}),
            (0x40071D, 0x400510, {"type": "call", "stmt_idx": DEFAULT_STATEMENT, "ins_addr": 0x400739}),
            (0x40071D, 0x40073E, {"type": "fake_return", "confirmed": True, "outside": False}),
            (0x40073E, 0x400530, {"type": "call", "stmt_idx": DEFAULT_STATEMENT, "ins_addr": 0x40074F}),
            (0x40073E, 0x400754, {"type": "fake_return", "confirmed": True, "outside": False}),
            # rejected() does not return
            (0x4007C9, 0x4006FD, {"type": "call", "stmt_idx": DEFAULT_STATEMENT, "ins_addr": 0x4007CE}),
            (0x4007C9, 0x4007D3, {"type": "fake_return", "outside": False}),
        ]
        for edge in edges:
            assert edge in main_g_edges

        # These tests fail for reasons of fastpath, probably
        # assert main.bp_on_stack
        # assert main.name == 'main'
        # assert main.retaddr_on_stack
        # assert 0x50 == main.sp_difference

        # TODO: Check the result returned
        # func_man.dbg_draw()

    def test_call_to(self):
        self.project.arch = ArchAMD64()

        self.project.kb.functions._add_call_to(0x400000, 0x400410, 0x400420, 0x400414)
        assert 0x400000 in self.project.kb.functions.keys()
        assert 0x400420 in self.project.kb.functions.keys()


if __name__ == "__main__":
    unittest.main()



from ailment.expression import BinaryOp, Const, Load

from ....utils.loader import is_pc, is_in_readonly_section, is_in_readonly_segment
from .base import PeepholeOptimizationExprBase


class SimplifyPcRelativeLoads(PeepholeOptimizationExprBase):
    """
    Simplifying pc-relative loads.
    """

    __slots__ = ()

    NAME = "Simplify PC-relative loads"
    expr_classes = (BinaryOp,)

    def optimize(self, expr: BinaryOp, **kwargs):
        # Load(addr) + pc ==> Const()
        if expr.op == "Add" and len(expr.operands) == 2 and isinstance(expr.operands[0], Load):
            op0, op1 = expr.operands

            if isinstance(op1, Const):
                # check if op1 is PC
                if hasattr(expr, "ins_addr") and is_pc(self.project, expr.ins_addr, op1.value):
                    # check if op0.addr points to a read-only section
                    addr = op0.addr.value
                    if is_in_readonly_section(self.project, addr) or is_in_readonly_segment(self.project, addr):
                        # found it!
                        # do the load first
                        try:
                            offset = self.project.loader.memory.unpack_word(addr, size=self.project.arch.bytes)
                        except KeyError:
                            return expr
                        value = offset + op1.value
                        return Const(None, None, value, self.project.arch.bits, **expr.tags)

        return expr



from io import BytesIO
from typing import List

import cle

from ...errors import AngrCorruptDBError
from ..models import DbObject


class LoaderSerializer:
    """
    Serialize/unserialize a CLE Loader object into/from an angr DB.
    """

    backend2name = {v: k for k, v in cle.ALL_BACKENDS.items()}

    @staticmethod
    def dump(session, loader):
        for obj in loader.all_objects:
            if isinstance(
                obj,
                (
                    cle.ExternObject,
                    cle.TLSObject,
                    cle.KernelObject,
                ),
            ):
                # skip dynamically created objects
                continue

            # does the object exist?
            exists = session.query(DbObject.id).filter_by(path=obj.binary).scalar() is not None
            if exists:
                # it exists. skip.
                continue

            # FIXME: We assume the binary and its libraries all still exist on the disk

            # save the object
            o = DbObject(
                main_object=loader.main_object is obj,
                path=obj.binary,
                content=open(obj.binary, "rb").read(),
                backend=LoaderSerializer.backend2name.get(obj.__class__),
                backend_args="",  # TODO: We will need support from CLE to store loader arguments
            )
            session.add(o)

    @staticmethod
    def load(session):
        all_objects = {}  # path to object
        main_object = None

        db_objects: List[DbObject] = session.query(DbObject)

        for db_o in db_objects:
            all_objects[db_o.path] = db_o
            if db_o.main_object:
                main_object = db_o

        if main_object is None:
            raise AngrCorruptDBError("Corrupt database: No main object.")

        # build params
        # FIXME: Load other objects

        loader = cle.Loader(
            BytesIO(main_object.content),
        )

        # fix the binary name of the main binary
        loader._main_binary_path = main_object.path
        loader.main_object.binary = main_object.path

        return loader



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.sim"  # pylint:disable=redefined-builtin

import logging
import os
import unittest

from archinfo import all_arches, ArchAMD64, ArchSoot

from angr.calling_conventions import DEFAULT_CC, default_cc, SimCCUnknown
from angr import SimState, sim_options as o, Project

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")

log = logging.getLogger(__name__)


class TestStackAlignment(unittest.TestCase):
    def test_alignment(self):
        for arch in all_arches:
            if arch.name in DEFAULT_CC and default_cc(arch.name, platform="Linux") is not SimCCUnknown:
                # There is nothing to test for soot about stack alignment
                if isinstance(arch, ArchSoot):
                    continue
                log.info("Testing stack alignment for %s", arch.name)
                st = SimState(arch=arch)
                cc = default_cc(arch.name, platform="Linux")(arch=arch)

                st.regs.sp = -1

                # setup callsite with one argument (0x1337), "returning" to 0
                cc.setup_callsite(st, 0, [0x1337], "void foo(int x)")

                # ensure stack alignment is correct
                assert st.solver.is_true((st.regs.sp + cc.STACKARG_SP_DIFF) % cc.STACK_ALIGNMENT == 0), (
                    "non-zero stack alignment after setup_callsite for %s" % cc
                )

    def test_sys_v_abi_compliance(self):
        arch = ArchAMD64()
        st = SimState(arch=arch)
        cc = default_cc(arch.name, platform="Linux")(arch=arch)

        st.regs.sp = -1

        # setup callsite with one argument (0x1337), "returning" to 0
        cc.setup_callsite(st, 0, [0x1337], "void foo(int x)")

        # (rsp+8) must be aligned to 16 as required by System V ABI.
        # ref: https://raw.githubusercontent.com/wiki/hjl-tools/x86-psABI/x86-64-psABI-1.0.pdf , page 18t
        assert st.solver.is_true((st.regs.rsp + 8) % 16 == 0), "System V ABI calling convention violated!"

    def test_initial_allocation(self):
        # not strictly about alignment but it's about stack initialization so whatever
        p = Project(os.path.join(test_location, "x86_64", "true"), auto_load_libs=False)
        s = p.factory.entry_state(add_options={o.STRICT_PAGE_ACCESS})
        s.memory.load(s.regs.sp - 0x10000, size=4)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3
__package__ = __package__ or "tests.serialization"  # pylint:disable=redefined-builtin

from contextlib import suppress
import gc
import os
import pickle
import shutil
import unittest

from claripy import BVS

import angr
from angr.storage import SimFile

from ..common import bin_location


test_location = os.path.join(bin_location, "tests")


class TestPickle(unittest.TestCase):
    @classmethod
    def tearDown(self):
        shutil.rmtree("pickletest", ignore_errors=True)
        shutil.rmtree("pickletest2", ignore_errors=True)
        with suppress(FileNotFoundError):
            os.remove("pickletest_good")
        with suppress(FileNotFoundError):
            os.remove("pickletest_bad")

    def _load_pickles(self):
        # This is the working case
        f = open("pickletest_good", "rb")
        f.close()

        # This will not work
        f = open("pickletest_bad", "rb")
        f.close()

    def _make_pickles(self):
        p = angr.Project(os.path.join(test_location, "i386", "fauxware"))

        fs = {
            "/dev/stdin": SimFile("/dev/stdin"),
            "/dev/stdout": SimFile("/dev/stdout"),
            "/dev/stderr": SimFile("/dev/stderr"),
        }

        MEM_SIZE = 1024
        mem_bvv = {}
        for f in fs:
            mem = BVS(f, MEM_SIZE * 8)
            mem_bvv[f] = mem

        f = open("pickletest_good", "wb")
        pickle.dump(mem_bvv, f, -1)
        f.close()

        # If you do not have a state you cannot write
        _ = p.factory.entry_state(fs=fs)
        for f in fs:
            mem = mem_bvv[f]
            fs[f].write(0, mem, MEM_SIZE)

        f = open("pickletest_bad", "wb")
        pickle.dump(mem_bvv, f, -1)
        f.close()

    def test_pickling(self):
        self._make_pickles()
        self._load_pickles()
        gc.collect()
        self._load_pickles()

    def test_project_pickling(self):
        # AnalysesHub should not be pickled together with the project itself
        p = angr.Project(os.path.join(test_location, "i386", "fauxware"))

        # make a copy of the active_preset so that we do not touch the global preset object. this is only for writing
        # this test case.
        p.analyses._active_preset = pickle.loads(pickle.dumps(p.analyses._active_preset, -1))
        assert len(p.analyses._active_preset._default_plugins) > 0
        p.analyses._active_preset = p.analyses._active_preset
        p.analyses._active_preset._default_plugins = {}
        assert len(p.analyses._active_preset._default_plugins) == 0

        s = pickle.dumps(p, -1)

        p1 = pickle.loads(s)
        assert len(p1.analyses._active_preset._default_plugins) > 0


if __name__ == "__main__":
    unittest.main()



# pylint:disable=missing-class-docstring,unused-argument
from collections import defaultdict
from typing import Optional, Any, Dict, Set, Tuple, Iterable, Union, DefaultDict, TYPE_CHECKING

import ailment
from ailment import Expression, Block, AILBlockWalker
from ailment.statement import Statement, Assignment, Call

from ..sequence_walker import SequenceWalker
from ..structuring.structurer_nodes import (
    ConditionNode,
    ConditionalBreakNode,
    LoopNode,
    CascadingConditionNode,
    SwitchCaseNode,
)

if TYPE_CHECKING:
    from angr.sim_variable import SimVariable
    from angr.knowledge_plugins.variables.variable_manager import VariableManagerInternal
    from ailment.expression import MultiStatementExpression


class LocationBase:
    __slots__ = ()


class StatementLocation(LocationBase):
    __slots__ = (
        "block_addr",
        "block_idx",
        "stmt_idx",
    )

    def __init__(self, block_addr, block_idx, stmt_idx):
        self.block_addr = block_addr
        self.block_idx = block_idx
        self.stmt_idx = stmt_idx

    def __repr__(self):
        return f"Loc: Statement@{self.block_addr:x}.{self.block_idx}-{self.stmt_idx}"

    def __hash__(self):
        return hash((StatementLocation, self.block_addr, self.block_idx, self.stmt_idx))

    def __eq__(self, other):
        return (
            isinstance(other, StatementLocation)
            and self.block_addr == other.block_addr
            and self.block_idx == other.block_idx
            and self.stmt_idx == other.stmt_idx
        )

    def copy(self):
        return StatementLocation(self.block_addr, self.block_idx, self.stmt_idx)


class ExpressionLocation(LocationBase):
    __slots__ = (
        "block_addr",
        "block_idx",
        "stmt_idx",
        "expr_idx",
    )

    def __init__(self, block_addr, block_idx, stmt_idx, expr_idx):
        self.block_addr = block_addr
        self.block_idx = block_idx
        self.stmt_idx = stmt_idx
        self.expr_idx = expr_idx

    def __repr__(self):
        return f"Loc: Expression@{self.block_addr:x}.{self.block_idx}-{self.stmt_idx}[{self.expr_idx}]"

    def statement_location(self) -> StatementLocation:
        return StatementLocation(self.block_addr, self.block_idx, self.stmt_idx)

    def __hash__(self):
        return hash((ExpressionLocation, self.block_addr, self.block_idx, self.stmt_idx, self.expr_idx))

    def __eq__(self, other):
        return (
            isinstance(other, ExpressionLocation)
            and self.block_addr == other.block_addr
            and self.block_idx == other.block_idx
            and self.stmt_idx == other.stmt_idx
            and self.expr_idx == other.expr_idx
        )


class ConditionLocation(LocationBase):
    __slots__ = (
        "node_addr",
        "case_idx",
    )

    def __init__(self, cond_node_addr, case_idx: Optional[int] = None):
        self.node_addr = cond_node_addr
        self.case_idx = case_idx

    def __repr__(self):
        return f"Loc: ConditionNode@{self.node_addr:x}.{self.case_idx}"

    def __hash__(self):
        return hash((ConditionLocation, self.node_addr, self.case_idx))

    def __eq__(self, other):
        return (
            isinstance(other, ConditionLocation)
            and self.node_addr == other.node_addr
            and self.case_idx == other.case_idx
        )


class ConditionalBreakLocation(LocationBase):
    __slots__ = ("node_addr",)

    def __init__(self, node_addr):
        self.node_addr = node_addr

    def __repr__(self):
        return f"Loc: ConditionalBreakNode@{self.node_addr:x}"

    def __hash__(self):
        return hash((ConditionalBreakLocation, self.node_addr))

    def __eq__(self, other):
        return isinstance(other, ConditionalBreakLocation) and self.node_addr == other.node_addr


class MultiStatementExpressionAssignmentFinder(AILBlockWalker):
    """
    Process statements in MultiStatementExpression objects and find assignments.
    """

    def __init__(self, stmt_handler):
        super().__init__()
        self._stmt_handler = stmt_handler

    def _handle_MultiStatementExpression(
        self, expr_idx, expr: "MultiStatementExpression", stmt_idx: int, stmt: Statement, block: Optional[Block]
    ):
        for idx, stmt_ in enumerate(expr.stmts):
            self._stmt_handler(idx, stmt_, block)
        return super()._handle_MultiStatementExpression(expr_idx, expr, stmt_idx, stmt, block)


class ExpressionUseFinder(AILBlockWalker):
    """
    Find where each variable is used.

    Additionally, determine if the expression being walked has load expressions inside. Such expressions can only be
    safely folded if there are no Store statements between the expression defining location and its use sites. For
    example, we can only safely fold variable assignments that use Load() when there are no Store()s between the
    assignment and its use site. Otherwise, the loaded expression may get updated later by a Store() statement.

    Here is a real AIL block:

    .. code-block:: none

        v16 = ((int)v23->field_5) + 1 & 255;
        v23->field_5 = ((char)(((int)v23->field_5) + 1 & 255));
        v13 = printf("Recieved packet %d for connection with %d\\n", v16, a0 & 255);

    In this case, folding v16 into the last printf() expression would be incorrect, since v23->field_5 is updated by
    the second statement.
    """

    __slots__ = (
        "uses",
        "has_load",
    )

    def __init__(self):
        super().__init__()
        self.uses: DefaultDict[SimVariable, Set[Tuple[Expression, Optional[ExpressionLocation]]]] = defaultdict(set)
        self.has_load = False

    def _handle_expr(
        self, expr_idx: int, expr: Expression, stmt_idx: int, stmt: Optional[Statement], block: Optional[Block]
    ) -> Any:
        if isinstance(expr, ailment.Register) and expr.variable is not None:
            if not (isinstance(stmt, ailment.Stmt.Assignment) and stmt.dst is expr):
                if block is not None:
                    self.uses[expr.variable].add((expr, ExpressionLocation(block.addr, block.idx, stmt_idx, expr_idx)))
                else:
                    self.uses[expr.variable].add((expr, None))
            return None
        return super()._handle_expr(expr_idx, expr, stmt_idx, stmt, block)

    def _handle_Load(
        self, expr_idx: int, expr: ailment.Expr.Load, stmt_idx: int, stmt: Statement, block: Optional[Block]
    ):
        self.has_load = True
        return super()._handle_Load(expr_idx, expr, stmt_idx, stmt, block)


class ExpressionCounter(SequenceWalker):
    """
    Find all expressions that are assigned once and only used once.
    """

    def __init__(self, node, variable_manager):
        handlers = {
            ConditionalBreakNode: self._handle_ConditionalBreak,
            ConditionNode: self._handle_Condition,
            LoopNode: self._handle_Loop,
            SwitchCaseNode: self._handle_SwitchCase,
            ailment.Block: self._handle_Block,
        }

        # each element in the set is a tuple of (source of the assignment statement, a tuple of unified variables that
        # the current assignment depends on, StatementLocation of the assignment statement, a Boolean variable that
        # indicates if ExpressionUseFinder has succeeded or not)
        self.assignments: DefaultDict[Any, Set[Tuple]] = defaultdict(set)
        self.uses: Dict[SimVariable, Set[Tuple[Expression, Optional[LocationBase]]]] = {}
        self._variable_manager: "VariableManagerInternal" = variable_manager

        super().__init__(handlers)
        self.walk(node)

    def _u(self, v) -> Optional["SimVariable"]:
        """
        Get unified variable for a given variable.
        """

        return self._variable_manager.unified_variable(v)

    def _handle_Statement(self, idx: int, stmt: ailment.Stmt, node: Union[ailment.Block, LoopNode]):
        if isinstance(stmt, ailment.Stmt.Assignment):
            if isinstance(stmt.dst, ailment.Expr.Register) and stmt.dst.variable is not None:
                u = self._u(stmt.dst.variable)
                if u is not None:
                    # dependency
                    dependency_finder = ExpressionUseFinder()
                    dependency_finder.walk_expression(stmt.src)
                    dependencies = tuple({self._u(v) for v in dependency_finder.uses})
                    self.assignments[u].add(
                        (
                            stmt.src,
                            dependencies,
                            StatementLocation(node.addr, node.idx if isinstance(node, ailment.Block) else None, idx),
                            dependency_finder.has_load,
                        )
                    )
        if (
            isinstance(stmt, ailment.Stmt.Call)
            and isinstance(stmt.ret_expr, ailment.Expr.Register)
            and stmt.ret_expr.variable is not None
        ):
            u = self._u(stmt.ret_expr.variable)
            if u is not None:
                dependency_finder = ExpressionUseFinder()
                dependency_finder.walk_expression(stmt)
                dependencies = tuple({self._u(v) for v in dependency_finder.uses})
                self.assignments[u].add(
                    (
                        stmt,
                        dependencies,
                        StatementLocation(node.addr, node.idx if isinstance(node, ailment.Block) else None, idx),
                        dependency_finder.has_load,
                    )
                )

    def _handle_Block(self, node: ailment.Block, **kwargs):
        # find assignments
        for idx, stmt in enumerate(node.statements):
            self._handle_Statement(idx, stmt, node)

        # walk the block and find uses of variables
        use_finder = ExpressionUseFinder()
        use_finder.walk(node)

        for v, content in use_finder.uses.items():
            u = self._u(v)
            if u is not None:
                if u not in self.uses:
                    self.uses[u] = set()
                self.uses[u] |= content

    def _collect_assignments(self, expr: ailment.Expr, node) -> None:
        finder = MultiStatementExpressionAssignmentFinder(self._handle_Statement)
        finder.walk_expression(expr, None, None, node)

    def _collect_uses(self, expr: Expression, loc: LocationBase):
        use_finder = ExpressionUseFinder()
        use_finder.walk_expression(expr, stmt_idx=-1)

        for var, uses in use_finder.uses.items():
            u = self._u(var)
            if u is None:
                continue
            for use in uses:
                if u not in self.uses:
                    self.uses[u] = set()
                self.uses[u].add((use[0], loc))

    def _handle_ConditionalBreak(self, node: ConditionalBreakNode, **kwargs):
        # collect uses on the condition expression
        self._collect_assignments(node.condition, node)
        self._collect_uses(node.condition, ConditionalBreakLocation(node.addr))
        return super()._handle_ConditionalBreak(node, **kwargs)

    def _handle_Condition(self, node: ConditionNode, **kwargs):
        # collect uses on the condition expression
        self._collect_assignments(node.condition, node)
        self._collect_uses(node.condition, ConditionLocation(node.addr))
        return super()._handle_Condition(node, **kwargs)

    def _handle_CascadingCondition(self, node: CascadingConditionNode, **kwargs):
        for idx, (condition, _) in enumerate(node.condition_and_nodes):
            self._collect_assignments(condition, node)
            self._collect_uses(condition, ConditionLocation(node.addr, idx))
        return super()._handle_CascadingCondition(node, **kwargs)

    def _handle_Loop(self, node: LoopNode, **kwargs):
        # collect uses on the condition expression
        if node.initializer is not None:
            self._collect_uses(node.initializer, ConditionLocation(node.addr))
        if node.iterator is not None:
            self._collect_uses(node.iterator, ConditionLocation(node.addr))
        if node.condition is not None:
            self._collect_assignments(node.condition, node)
            self._collect_uses(node.condition, ConditionLocation(node.addr))
        return super()._handle_Loop(node, **kwargs)

    def _handle_SwitchCase(self, node: SwitchCaseNode, **kwargs):
        self._collect_uses(node.switch_expr, ConditionLocation(node.addr))
        return super()._handle_SwitchCase(node, **kwargs)


class ExpressionReplacer(AILBlockWalker):
    def __init__(self, assignments: Dict, uses: Dict, variable_manager):
        super().__init__()
        self._assignments = assignments
        self._uses = uses
        self._variable_manager: "VariableManagerInternal" = variable_manager

    def _u(self, v) -> Optional["SimVariable"]:
        """
        Get unified variable for a given variable.
        """
        return self._variable_manager.unified_variable(v)

    def _handle_MultiStatementExpression(
        self, expr_idx, expr: "MultiStatementExpression", stmt_idx: int, stmt: Statement, block: Optional[Block]
    ):
        changed = False
        new_statements = []
        for idx, stmt_ in enumerate(expr.stmts):
            if (
                isinstance(stmt_, Assignment)
                and isinstance(stmt_.dst, ailment.Expr.Register)
                and stmt_.dst.variable is not None
            ):
                if stmt_.dst.variable in self._assignments:
                    # remove this statement
                    changed = True
                    continue

            new_stmt = self._handle_stmt(idx, stmt_, None)
            if new_stmt is not None and new_stmt is not stmt_:
                changed = True
                if isinstance(new_stmt, Assignment) and new_stmt.src.likes(new_stmt.dst):
                    # this statement is simplified into reg = reg. ignore it
                    continue
                new_statements.append(new_stmt)
            else:
                new_statements.append(stmt_)

        new_expr = self._handle_expr(0, expr.expr, stmt_idx, stmt, block)
        if new_expr is not None and new_expr is not expr.expr:
            changed = True
        else:
            new_expr = expr.expr

        if changed:
            if not new_statements:
                # it is no longer a multi-statement expression
                return new_expr
            expr_ = expr.copy()
            expr_.expr = new_expr
            expr_.stmts = new_statements
            return expr_
        return None

    def _handle_Assignment(self, stmt_idx: int, stmt: Assignment, block: Optional[Block]):
        # override the base handler and make sure we do not replace .dst with a Call expression
        changed = False

        dst = self._handle_expr(0, stmt.dst, stmt_idx, stmt, block)
        if dst is not None and dst is not stmt.dst and not isinstance(dst, Call):
            changed = True
        else:
            dst = stmt.dst

        src = self._handle_expr(1, stmt.src, stmt_idx, stmt, block)
        if src is not None and src is not stmt.src:
            changed = True
        else:
            src = stmt.src

        if changed:
            new_stmt = Assignment(stmt.idx, dst, src, **stmt.tags)
            if block is not None:
                # update the statement directly in the block
                block.statements[stmt_idx] = new_stmt
            return new_stmt
        return None

    def _handle_expr(
        self, expr_idx: int, expr: Expression, stmt_idx: int, stmt: Optional[Statement], block: Optional[Block]
    ) -> Any:
        if isinstance(expr, ailment.Register) and expr.variable is not None:
            unified_var = self._u(expr.variable)
            if unified_var in self._uses:
                replace_with, _ = self._assignments[unified_var]
                return replace_with
        return super()._handle_expr(expr_idx, expr, stmt_idx, stmt, block)


class ExpressionFolder(SequenceWalker):
    def __init__(self, assignments: Dict, uses: Dict, node, variable_manager):
        handlers = {
            ailment.Block: self._handle_Block,
            ConditionNode: self._handle_Condition,
            ConditionalBreakNode: self._handle_ConditionalBreak,
            SwitchCaseNode: self._handle_SwitchCase,
        }

        super().__init__(handlers)
        self._assignments = assignments
        self._uses = uses
        self._variable_manager = variable_manager
        self.walk(node)

    def _u(self, v) -> Optional["SimVariable"]:
        """
        Get unified variable for a given variable.
        """
        return self._variable_manager.unified_variable(v)

    def _handle_Block(self, node: ailment.Block, **kwargs):
        # Walk the block to remove each assignment and replace uses of each variable
        new_stmts = []
        for stmt in node.statements:
            if isinstance(stmt, ailment.Stmt.Assignment):
                if isinstance(stmt.dst, ailment.Expr.Register) and stmt.dst.variable is not None:
                    if stmt.dst.variable in self._assignments:
                        # remove this statement
                        continue
            if (
                isinstance(stmt, ailment.Stmt.Call)
                and isinstance(stmt.ret_expr, ailment.Expr.Register)
                and stmt.ret_expr.variable is not None
            ):
                unified_var = self._u(stmt.ret_expr.variable)
                if unified_var in self._assignments:
                    # remove this statement
                    continue
            new_stmts.append(stmt)
        node.statements = new_stmts

        # Walk the block to replace the use of each variable
        replacer = ExpressionReplacer(self._assignments, self._uses, self._variable_manager)
        replacer.walk(node)

    def _handle_ConditionalBreak(self, node: ConditionalBreakNode, **kwargs):
        replacer = ExpressionReplacer(self._assignments, self._uses, self._variable_manager)
        r = replacer.walk_expression(node.condition)
        if r is not None and r is not node.condition:
            node.condition = r
        return super()._handle_ConditionalBreak(node, **kwargs)

    def _handle_Condition(self, node: ConditionNode, **kwargs):
        replacer = ExpressionReplacer(self._assignments, self._uses, self._variable_manager)
        r = replacer.walk_expression(node.condition)
        if r is not None and r is not node.condition:
            node.condition = r
        return super()._handle_Condition(node, **kwargs)

    def _handle_CascadingCondition(self, node: CascadingConditionNode, **kwargs):
        replacer = ExpressionReplacer(self._assignments, self._uses, self._variable_manager)
        for idx in range(len(node.condition_and_nodes)):  # pylint:disable=consider-using-enumerate
            cond, _ = node.condition_and_nodes[idx]
            r = replacer.walk_expression(cond)
            if r is not None and r is not cond:
                node.condition_and_nodes[idx] = (r, node.condition_and_nodes[idx][1])
        return super()._handle_CascadingCondition(node, **kwargs)

    def _handle_Loop(self, node: LoopNode, **kwargs):
        replacer = ExpressionReplacer(self._assignments, self._uses, self._variable_manager)

        # iterator
        if node.iterator is not None:
            r = replacer.walk_expression(node.iterator)
            if r is not None and r is not node.iterator:
                node.iterator = r

        # initializer
        if node.initializer is not None:
            r = replacer.walk_expression(node.initializer)
            if r is not None and r is not node.initializer:
                node.initializer = r

        # condition
        if node.condition is not None:
            r = replacer.walk_expression(node.condition)
            if r is not None and r is not node.condition:
                node.condition = r

        return super()._handle_Loop(node, **kwargs)

    def _handle_SwitchCase(self, node: SwitchCaseNode, **kwargs):
        replacer = ExpressionReplacer(self._assignments, self._uses, self._variable_manager)

        r = replacer.walk_expression(node.switch_expr)
        if r is not None and r is not node.switch_expr:
            node.switch_expr = r

        return super()._handle_SwitchCase(node, **kwargs)


class StoreStatementFinder(SequenceWalker):
    """
    Determine if there are any Store statements between two given statements.

    This class overrides _handle_Sequence() and _handle_MultiNode() to ensure they traverse nodes from top to bottom.
    """

    def __init__(self, node, intervals: Iterable[Tuple[StatementLocation, LocationBase]]):
        handlers = {
            ConditionNode: self._handle_Condition,
            CascadingConditionNode: self._handle_CascadingCondition,
            ConditionalBreakNode: self._handle_ConditionalBreak,
            ailment.Block: self._handle_Block,
        }

        self._intervals = intervals

        self._start_to_ends: DefaultDict[StatementLocation, Set[LocationBase]] = defaultdict(set)
        self._end_to_starts: DefaultDict[LocationBase, Set[StatementLocation]] = defaultdict(set)
        self.interval_to_hasstore: Dict[Tuple[StatementLocation, StatementLocation], bool] = {}
        for start, end in intervals:
            self._start_to_ends[start].add(end)
            self._end_to_starts[end].add(start)

        self._active_intervals = set()

        super().__init__(handlers)
        self.walk(node)

    def _handle_Sequence(self, node, **kwargs):
        i = 0
        while i < len(node.nodes):
            node_ = node.nodes[i]
            self._handle(node_, parent=node, index=i)
            i += 1

    def _handle_MultiNode(self, node, **kwargs):
        i = 0
        while i < len(node.nodes):
            node_ = node.nodes[i]
            self._handle(node_, parent=node, index=i)
            i += 1

    def _handle_Block(self, node: ailment.Block, **kwargs):
        stmt_loc = StatementLocation(node.addr, node.idx, None)
        for idx, stmt in enumerate(node.statements):
            stmt_loc.stmt_idx = idx
            if stmt_loc in self._start_to_ends:
                for end in self._start_to_ends[stmt_loc]:
                    self._active_intervals.add((stmt_loc.copy(), end))
            if stmt_loc in self._end_to_starts:
                for start in self._end_to_starts[stmt_loc]:
                    self._active_intervals.discard((start, stmt_loc))
            if isinstance(stmt, ailment.Stmt.Store):
                for interval in self._active_intervals:
                    self.interval_to_hasstore[interval] = True

    def _handle_Condition(self, node, **kwargs):
        cond_loc = ConditionLocation(node.addr)
        if cond_loc in self._end_to_starts:
            for start in self._end_to_starts[cond_loc]:
                self._active_intervals.discard((start, cond_loc))
        super()._handle_Condition(node, **kwargs)

    def _handle_CascadingCondition(self, node: CascadingConditionNode, **kwargs):
        cond_loc = ConditionLocation(node.addr, None)
        for idx in range(len(node.condition_and_nodes)):
            cond_loc.case_idx = idx
            if cond_loc in self._end_to_starts[cond_loc]:
                for start in self._end_to_starts[cond_loc]:
                    self._active_intervals.discard((start, cond_loc))
        super()._handle_CascadingCondition(node, **kwargs)

    def _handle_ConditionalBreak(self, node: ConditionalBreakNode, **kwargs):
        cond_break_loc = ConditionalBreakLocation(node.addr)
        if cond_break_loc in self._end_to_starts:
            for start in self._end_to_starts[cond_break_loc]:
                self._active_intervals.discard((start, cond_break_loc))
        super()._handle_ConditionalBreak(node, **kwargs)

    def has_store(self, start: StatementLocation, end: StatementLocation) -> bool:
        return self.interval_to_hasstore.get((start, end), False)



#!/usr/bin/env python3
# pylint: disable=missing-class-docstring,no-self-use,line-too-long
__package__ = __package__ or "tests.analyses.decompiler"  # pylint:disable=redefined-builtin

import os
import unittest

import networkx

import angr
import angr.analyses.decompiler
from angr.analyses.decompiler.structuring import DreamStructurer

from ...common import bin_location


test_location = os.path.join(bin_location, "tests")


class DummyNode:
    def __init__(self, n):
        self.n = n

    def __hash__(self):
        return hash(self.n)

    def __eq__(self, other):
        return isinstance(other, DummyNode) and self.n == other.n or isinstance(other, int) and self.n == other

    @property
    def addr(self):
        return self.n

    def __repr__(self):
        return "<Node %d>" % self.n


def d(n):
    return DummyNode(n)


def D(*edge):
    return DummyNode(edge[0]), DummyNode(edge[1])


class TestStructurer(unittest.TestCase):
    def test_region_identifier_0(self):
        g = networkx.DiGraph()

        #
        #       1
        #       |
        #       2
        #      / \
        #     3  4
        #     \  /
        #      5
        #      |
        #      6

        g.add_edges_from(
            [
                D(1, 2),
                D(2, 3),
                D(2, 4),
                D(3, 5),
                D(4, 5),
                D(5, 6),
            ]
        )

        ri = angr.analyses.decompiler.RegionIdentifier(None, graph=g)
        region = ri.region
        assert len(region.graph.nodes()) == 2

    def test_region_identifier_1(self):
        g = networkx.DiGraph()

        #
        #        1
        #        |
        #        2
        #        | \
        #        | 3
        #        | /
        #        4
        #        |
        #        5
        #        | \
        #        | 6
        #        | /
        #        7
        #        |
        #        8

        g.add_edges_from(
            [
                D(1, 2),
                D(2, 3),
                D(3, 4),
                D(2, 4),
                D(4, 5),
                D(5, 6),
                D(6, 7),
                D(5, 7),
                D(7, 8),
            ]
        )

        ri = angr.analyses.decompiler.RegionIdentifier(None, graph=g)
        region = ri.region
        assert len(region.graph.nodes()) == 2

    def test_smoketest(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "all"), auto_load_libs=False, load_debug_info=True)
        cfg = p.analyses.CFG(data_references=True, normalize=True)

        main_func = cfg.kb.functions["main"]

        # convert function blocks to AIL blocks
        clinic = p.analyses.Clinic(main_func)

        # recover regions
        ri = p.analyses.RegionIdentifier(main_func, graph=clinic.graph)

        # structure it
        st = p.analyses[DreamStructurer].prep()(ri.region)

        # simplify it
        _ = p.analyses.RegionSimplifier(main_func, st.result)

    def test_smoketest_cm3_firmware(self):
        p = angr.Project(
            os.path.join(test_location, "armel", "i2c_master_read-nucleol152re.elf"),
            auto_load_libs=False,
            load_debug_info=True,
        )
        cfg = p.analyses.CFG(normalize=True, force_complete_scan=False)

        main_func = cfg.kb.functions["main"]

        # convert function blocks to AIL blocks
        clinic = p.analyses.Clinic(main_func)

        # recover regions
        ri = p.analyses.RegionIdentifier(main_func, graph=clinic.graph)

        # structure it
        p.analyses[DreamStructurer].prep()(ri.region)

    def test_simple(self):
        p = angr.Project(os.path.join(test_location, "x86_64", "all"), auto_load_libs=False, load_debug_info=True)
        cfg = p.analyses.CFG(data_references=True, normalize=True)

        main_func = cfg.kb.functions["main"]

        # convert function blocks to AIL blocks
        clinic = p.analyses.Clinic(main_func)

        # recover regions
        ri = p.analyses.RegionIdentifier(main_func, graph=clinic.graph)

        # structure it
        rs = p.analyses.RecursiveStructurer(ri.region)

        # simplify it
        s = p.analyses.RegionSimplifier(main_func, rs.result)

        codegen = p.analyses.StructuredCodeGenerator(main_func, s.result, cfg=cfg, ail_graph=clinic.graph)
        print(codegen.text)

    def test_simple_loop(self):
        p = angr.Project(
            os.path.join(test_location, "x86_64", "cfg_loop_unrolling"), auto_load_libs=False, load_debug_info=True
        )
        cfg = p.analyses.CFG(data_references=True, normalize=True)

        test_func = cfg.kb.functions["test_func"]

        # convert function blocks to AIL blocks
        clinic = p.analyses.Clinic(test_func)

        # recover regions
        ri = p.analyses.RegionIdentifier(test_func, graph=clinic.graph)

        # structure it
        rs = p.analyses.RecursiveStructurer(ri.region)

        # simplify it
        s = p.analyses.RegionSimplifier(test_func, rs.result)

        codegen = p.analyses.StructuredCodeGenerator(test_func, s.result, cfg=cfg, ail_graph=clinic.graph)
        print(codegen.text)

        assert len(codegen.map_pos_to_node._posmap) > 1
        assert len(codegen.map_ast_to_pos) > 1

    def test_recursive_structuring(self):
        p = angr.Project(
            os.path.join(test_location, "x86_64", "cfg_loop_unrolling"), auto_load_libs=False, load_debug_info=True
        )
        cfg = p.analyses.CFG(data_references=True, normalize=True)

        test_func = cfg.kb.functions["test_func"]

        # convert function blocks to AIL blocks
        clinic = p.analyses.Clinic(test_func)

        # recover regions
        ri = p.analyses.RegionIdentifier(test_func, graph=clinic.graph)

        # structure it
        rs = p.analyses.RecursiveStructurer(ri.region)

        # simplify it
        s = p.analyses.RegionSimplifier(test_func, rs.result)

        codegen = p.analyses.StructuredCodeGenerator(test_func, s.result, cfg=cfg, ail_graph=clinic.graph)
        print(codegen.text)

    def test_while_true_break(self):
        p = angr.Project(
            os.path.join(test_location, "x86_64", "test_decompiler_loops_O0"),
            auto_load_libs=False,
            load_debug_info=True,
        )
        cfg = p.analyses.CFG(data_references=True, normalize=True)

        test_func = cfg.kb.functions["_while_true_break"]

        # convert function blocks to AIL blocks
        clinic = p.analyses.Clinic(test_func)

        # recover regions
        ri = p.analyses.RegionIdentifier(test_func, graph=clinic.graph)

        # structure it
        rs = p.analyses.RecursiveStructurer(ri.region)

        # simplify it
        s = p.analyses.RegionSimplifier(test_func, rs.result)

        codegen = p.analyses.StructuredCodeGenerator(test_func, s.result, cfg=cfg, ail_graph=clinic.graph)

        print(codegen.text)

    def test_while(self):
        p = angr.Project(
            os.path.join(test_location, "x86_64", "test_decompiler_loops_O0"),
            auto_load_libs=False,
            load_debug_info=True,
        )
        cfg = p.analyses.CFG(data_references=True, normalize=True)

        test_func = cfg.kb.functions["_while"]

        # convert function blocks to AIL blocks
        clinic = p.analyses.Clinic(test_func)

        # recover regions
        ri = p.analyses.RegionIdentifier(test_func, graph=clinic.graph)

        # structure it
        rs = p.analyses.RecursiveStructurer(ri.region)

        # simplify it
        s = p.analyses.RegionSimplifier(test_func, rs.result)

        codegen = p.analyses.StructuredCodeGenerator(test_func, s.result, cfg=cfg, ail_graph=clinic.graph)

        print(codegen.text)


if __name__ == "__main__":
    unittest.main()



import angr

# pylint:disable=arguments-differ


class getpid(angr.SimProcedure):
    def run(self):
        return self.state.posix.pid


class getppid(angr.SimProcedure):
    def run(self):
        return self.state.posix.ppid



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("ksuser.dll")
prototypes = \
    {
        #
        'KsCreateAllocator': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"Anonymous1": SimUnion({"OptionsFlags": SimTypeInt(signed=False, label="UInt32"), "RequirementsFlags": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "PoolType": SimTypeInt(signed=False, label="UInt32"), "Frames": SimTypeInt(signed=False, label="UInt32"), "FrameSize": SimTypeInt(signed=False, label="UInt32"), "Anonymous2": SimUnion({"FileAlignment": SimTypeInt(signed=False, label="UInt32"), "FramePitch": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None"), "Reserved": SimTypeInt(signed=False, label="UInt32")}, name="KSALLOCATOR_FRAMING", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ConnectionHandle", "AllocatorFraming", "AllocatorHandle"]),
        #
        'KsCreateClock': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"CreateFlags": SimTypeInt(signed=False, label="UInt32")}, name="KSCLOCK_CREATE", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ConnectionHandle", "ClockCreate", "ClockHandle"]),
        #
        'KsCreatePin': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"Interface": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"Set": SimTypeBottom(label="Guid"), "Id": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")}, name="KSIDENTIFIER", pack=False, align=None), "Medium": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"Set": SimTypeBottom(label="Guid"), "Id": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")}, name="KSIDENTIFIER", pack=False, align=None), "PinId": SimTypeInt(signed=False, label="UInt32"), "PinToHandle": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Priority": SimStruct({"PriorityClass": SimTypeInt(signed=False, label="UInt32"), "PrioritySubClass": SimTypeInt(signed=False, label="UInt32")}, name="KSPRIORITY", pack=False, align=None)}, name="KSPIN_CONNECT", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["FilterHandle", "Connect", "DesiredAccess", "ConnectionHandle"]),
        #
        'KsCreateTopologyNode': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"CreateFlags": SimTypeInt(signed=False, label="UInt32"), "Node": SimTypeInt(signed=False, label="UInt32")}, name="KSNODE_CREATE", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ParentHandle", "NodeCreate", "DesiredAccess", "NodeHandle"]),
        #
        'KsCreateAllocator2': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"Anonymous1": SimUnion({"OptionsFlags": SimTypeInt(signed=False, label="UInt32"), "RequirementsFlags": SimTypeInt(signed=False, label="UInt32")}, name="<anon>", label="None"), "PoolType": SimTypeInt(signed=False, label="UInt32"), "Frames": SimTypeInt(signed=False, label="UInt32"), "FrameSize": SimTypeInt(signed=False, label="UInt32"), "Anonymous2": SimUnion({"FileAlignment": SimTypeInt(signed=False, label="UInt32"), "FramePitch": SimTypeInt(signed=True, label="Int32")}, name="<anon>", label="None"), "Reserved": SimTypeInt(signed=False, label="UInt32")}, name="KSALLOCATOR_FRAMING", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ConnectionHandle", "AllocatorFraming", "AllocatorHandle"]),
        #
        'KsCreateClock2': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"CreateFlags": SimTypeInt(signed=False, label="UInt32")}, name="KSCLOCK_CREATE", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ConnectionHandle", "ClockCreate", "ClockHandle"]),
        #
        'KsCreatePin2': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"Interface": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"Set": SimTypeBottom(label="Guid"), "Id": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")}, name="KSIDENTIFIER", pack=False, align=None), "Medium": SimStruct({"Anonymous": SimUnion({"Anonymous": SimStruct({"Set": SimTypeBottom(label="Guid"), "Id": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="UInt32")}, name="_Anonymous_e__Struct", pack=False, align=None), "Alignment": SimTypeLongLong(signed=True, label="Int64")}, name="<anon>", label="None")}, name="KSIDENTIFIER", pack=False, align=None), "PinId": SimTypeInt(signed=False, label="UInt32"), "PinToHandle": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "Priority": SimStruct({"PriorityClass": SimTypeInt(signed=False, label="UInt32"), "PrioritySubClass": SimTypeInt(signed=False, label="UInt32")}, name="KSPRIORITY", pack=False, align=None)}, name="KSPIN_CONNECT", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["FilterHandle", "Connect", "DesiredAccess", "ConnectionHandle"]),
        #
        'KsCreateTopologyNode2': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"CreateFlags": SimTypeInt(signed=False, label="UInt32"), "Node": SimTypeInt(signed=False, label="UInt32")}, name="KSNODE_CREATE", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["ParentHandle", "NodeCreate", "DesiredAccess", "NodeHandle"]),
    }

lib.set_prototypes(prototypes)



from .functions import FunctionManager, Function
from .variables import VariableManager
from .debug_variables import DebugVariableManager
from .comments import Comments
from .data import Data
from .indirect_jumps import IndirectJumps
from .labels import Labels
from .cfg import CFGManager
from .xrefs import XRefManager
from .plugin import KnowledgeBasePlugin
from .sync import SyncController
from .patches import PatchManager
from .key_definitions import KeyDefinitionManager
from .propagations import PropagationManager
from .structured_code import StructuredCodeManager
from .types import TypesStore
from .callsite_prototypes import CallsitePrototypes



import logging

from ..expressions.invoke import InvokeBase
from .base import SimSootStmt

l = logging.getLogger("angr.engines.soot.statements.assign")


class SimSootStmt_Assign(SimSootStmt):
    def _execute(self):
        dst = self._translate_value(self.stmt.left_op)
        src_expr = self._translate_expr(self.stmt.right_op)
        if isinstance(src_expr, InvokeBase):
            # right hand side of the the assignment is an invocation
            # => The assumption is that if the src_expr contains an invoke, it
            #    is always just that invoke. In other words, the only possible
            #    form of "invoke in assignment" is: reg = invoke.
            #    This requirement *should* be enforced by the lifting to Soot IR.
            # => We deal with the invoke assignment, by
            #    1) saving the destination variable
            #    2) executing the function
            #    3) assign the return value to the destination variables
            #       after the function returns
            self._add_invoke_target(invoke_expr=src_expr, ret_var=dst)
            # exit prematurely
            return
        src_val = src_expr.expr
        l.debug("Assign %r := %r", dst, src_val)
        self.state.memory.store(dst, src_val)
